/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2007 Aurelien Jarno <aurelien@aurel32.net>
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/spinlock.h>
#include <linux/gpio.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
//#include "bspchip.h"
#include <asm/mach-realtek/bspchip.h>

static unsigned int rtl819x_gpio_mux(u32 pin, u32 *value, u32 *address );

static DEFINE_SPINLOCK(rtl819x_gpio_lock);

static struct timer_list watchdog_timer;
static char en_wdg_pulse='1';

static int __rtl819x_gpio_get_value(unsigned gpio)
{
	unsigned int data;

	data = (__raw_readl((void __iomem*)BSP_GPIO_DAT_REG(gpio)) >> BSP_GPIO_BIT(gpio) ) & 1;

	return data;
}

static int rtl819x_gpio_get_value(struct gpio_chip *chip, unsigned pin)
{
	if (pin >= BSP_GPIO_PIN_MAX)
		return -1;

	return __rtl819x_gpio_get_value(pin);
}

static int __rtl819x_gpio_set_value(unsigned pin, int value)
{
	unsigned int data;

	data = __raw_readl((void __iomem*)BSP_GPIO_DAT_REG(pin));

	#if 0
	printk("%s(%d)xxx 0x%08x\n",__FUNCTION__,__LINE__,
		__raw_readl((void __iomem*)PINMUX_SEL_REG(_ata_rtl8972d_v1xx_leds_gpio[i].gpio)));
	printk("%s(%d)xxx 0x%08x\n",__FUNCTION__,__LINE__,
		__raw_readl((void __iomem*)BSP_GPIO_CNR_REG(_ata_rtl8972d_v1xx_leds_gpio[i].gpio)));
	printk("%s(%d)xxx 0x%08x\n",__FUNCTION__,__LINE__,
		__raw_readl((void __iomem*)BSP_GPIO_DIR_REG(_ata_rtl8972d_v1xx_leds_gpio[i].gpio)));

	printk("%s(%d)xxx pin=%d, val=%d\n",__FUNCTION__,__LINE__,pin,value);
	printk("%s(%d)xxx 0x%08x\n",__FUNCTION__,__LINE__,REG32(BSP_GPIO_DAT_REG(pin)));
	#endif

	if (value == 0)
		data &= ~(1 << BSP_GPIO_BIT(pin));
	else
		data |= (1 << BSP_GPIO_BIT(pin));

	//printk("%s(%d)yyy 0x%08x\n",__FUNCTION__,__LINE__,data);

	__raw_writel(data, (void __iomem*)BSP_GPIO_DAT_REG(pin));

	//printk("%s(%d)xxx 0x%08x\n",__FUNCTION__,__LINE__,
	//	__raw_readl((void __iomem*)BSP_GPIO_DAT_REG(pin)));

	return 0;
}

static void rtl819x_gpio_set_value(struct gpio_chip *chip,
				  unsigned pin, int value)
{
	if (pin >= BSP_GPIO_PIN_MAX)
		return;

	__rtl819x_gpio_set_value(pin, value);
}

static int rtl819x_gpio_direction_input(struct gpio_chip *chip,
				       unsigned pin)
{
	unsigned long flags;

	if (pin >= BSP_GPIO_PIN_MAX)
		return -1;

	spin_lock_irqsave(&rtl819x_gpio_lock, flags);

	/* 0 : input */
	__raw_writel(__raw_readl((void __iomem*)BSP_GPIO_DIR_REG(pin)) & ~(1 << BSP_GPIO_BIT(pin)), 
					(void __iomem*)BSP_GPIO_DIR_REG(pin));

	spin_unlock_irqrestore(&rtl819x_gpio_lock, flags);

	return 0;
}

static int rtl819x_gpio_direction_output(struct gpio_chip *chip,
					unsigned pin, int value)
{
	unsigned long flags;

	if (pin >= BSP_GPIO_PIN_MAX)
		return -1;

	spin_lock_irqsave(&rtl819x_gpio_lock, flags);

	__raw_writel(__raw_readl((void __iomem*)BSP_GPIO_DIR_REG(pin)) | (1 << BSP_GPIO_BIT(pin)), 
					(void __iomem*)BSP_GPIO_DIR_REG(pin) );

	spin_unlock_irqrestore(&rtl819x_gpio_lock, flags);

	return 0;
}


static struct gpio_chip rtl819x_gpio_peripheral = {

	.label				= "rtl819x_gpio",
	.get				= rtl819x_gpio_get_value,
	.set				= rtl819x_gpio_set_value,
	.direction_input	= rtl819x_gpio_direction_input,
	.direction_output	= rtl819x_gpio_direction_output,
	.base				= 0,
};

void rtl819x_gpio_pin_enable(u32 pin)
{
	unsigned long flags;
	unsigned int mask  = 0;
	unsigned int mux =0;
	unsigned int mux_reg;
	unsigned int val;
	


    if (pin >= TOTAL_PIN_MAX)
		return;

	spin_lock_irqsave(&rtl819x_gpio_lock, flags);

	/* pin MUX1 */
	mask = rtl819x_gpio_mux(pin,&val,&mux_reg);

	//mux  = __raw_readl((void __iomem*) BSP_PINMUX_SEL_REG(pin));
	mux  = __raw_readl((void __iomem*)mux_reg);

	//if (mask != 0 && (mux & mask) == 0)
	if (mask != 0)
	    __raw_writel( ((mux&(~mask)) | (val)), (void __iomem*)mux_reg);

	/* 0 as BSP_GPIO pin */
    if(pin < BSP_GPIO_PIN_MAX)
    {
        __raw_writel(__raw_readl((void __iomem*)BSP_GPIO_CNR_REG(pin)) & ~(1<<BSP_GPIO_BIT(pin)), 
					(void __iomem*)BSP_GPIO_CNR_REG(pin));
    }

	spin_unlock_irqrestore(&rtl819x_gpio_lock, flags);

}

void rtl819x_gpio_pin_disable(u32 pin)
{
	unsigned long flags;

	spin_lock_irqsave(&rtl819x_gpio_lock, flags);

	/* 1 as peripheral pin */
	__raw_writel(__raw_readl((void __iomem*)BSP_GPIO_CNR_REG(pin)) | (1<<BSP_GPIO_BIT(pin)), 
					(void __iomem*)BSP_GPIO_CNR_REG(pin));

	spin_unlock_irqrestore(&rtl819x_gpio_lock, flags);
}

void rtl819x_gpio_pin_set_val(u32 pin, int val)
{
	__rtl819x_gpio_set_value(pin, val);
}

int rtl819x_gpio_pin_get_val(u32 pin)
{
	return __rtl819x_gpio_get_value(pin);
}
//--------------------------------------------------------
////  Watchdog Pulse
////--------------------------------------------------------
static void watchdog_function(unsigned long data)
{
  if(en_wdg_pulse == '1') {
      if (watchdog_timer.data==0)
      {
          rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_E1, 1);
          watchdog_timer.data=1;
      } else {
          rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_E1, 0);
          watchdog_timer.data=0;
      }
  }

  mod_timer(&watchdog_timer, jiffies + 100);
}


//----------------------------------------------
// Enable usb power
//----------------------------------------------
static int usb_power_read_proc(struct seq_file *s, void *v)
{
        switch (rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G0))
        {
                case 1:
                        seq_printf(s,"usb_power: 0 \n"); //low active
                        break;
                case 0:
                        seq_printf(s,"usb_power: 1 \n");
                        break;
                default:
                        seq_printf(s,"usb_power G0: %d\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G0));
                        break;
        }
        return 0;
}

static int usb_power_write_proc(struct file *file, const char *buffer,
                unsigned long count, void *data)
{
        char usb_power;

        if (count < 2)
                return -EFAULT;

        if (buffer && !copy_from_user(&usb_power, buffer, 1)) {
                switch(usb_power)
                {
                        case '1':
                                rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G0, 0); //low active
                                break;
                        case '0':
                                rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G0, 1);
                                break;
                        case 'h':
                                printk("----------------------\n");
                                printk("usb_power: Help Menu\n");
                                printk("----------------------\n");
                                printk("1: Enable USB Power\n");
                                printk("0: Disable USB Power\n");
                                printk("h: Show this Help Menu\n");
                                break;
                        default:
                                printk("Invaild Value!\n");
                                return -EFAULT;
                }
                return count;
        }
        return -EFAULT;
}

int usb_power_single_open(struct inode *inode, struct file *file)
{
        return(single_open(file, usb_power_read_proc, NULL));
}

static ssize_t usb_power_single_write(struct file * file, const char __user * userbuf,
             size_t count, loff_t * off)
{
    return usb_power_write_proc(file, userbuf,count, off);
}

static const struct file_operations usb_power_proc_fops = {
        .open           = usb_power_single_open,
        .write          = usb_power_single_write,
        .read           = seq_read,
        .llseek         = seq_lseek,
        .release        = single_release,
};

static int en_wdg_pulse_read_proc(struct seq_file *s, void *v)
{
    seq_printf(s,"en_wdg_pulse: %c\n", en_wdg_pulse);

    return 0;
}

static int en_wdg_pulse_write_proc(struct file *file, const char *buffer,
                unsigned long count, void *data)
{
    if (count < 2)
        return -EFAULT;

    if (buffer && !copy_from_user(&en_wdg_pulse, buffer, 1)) {
        switch(en_wdg_pulse)
        {
            case '1':
            case '0':
                    break;
            case 'd':
    en_wdg_pulse='1';
    printk("REG32(0xb8000008): 0x%08x\n\n",REG32(0xb8000008));
    printk("PIN_MUX_SEL_1(100):     0x%08x\n",  REG32(0xb8000100));
    printk("PIN_MUX_SEL_2(104):     0x%08x\n",  REG32(0xb8000104));
    printk("PIN_MUX_SEL_3(108):     0x%08x\n",  REG32(0xb8000108));
    printk("PIN_MUX_SEL_4(10C):     0x%08x\n\n",REG32(0xb800010c));
    printk("BSP_PABCD_CNR     :     0x%08x\n",  REG32(BSP_PABCD_CNR));
    printk("BSP_PABCD_DIR     :     0x%08x\n",  REG32(BSP_PABCD_DIR));
    printk("BSP_PABCD_DAT     :     0x%08x\n",  REG32(BSP_PABCD_DAT));
    printk("BSP_PEFGH_CNR     :     0x%08x\n",  REG32(BSP_PEFGH_CNR));
    printk("BSP_PEFGH_DIR     :     0x%08x\n",  REG32(BSP_PEFGH_DIR));
    printk("BSP_PEFGH_DAT     :     0x%08x\n\n",REG32(BSP_PEFGH_DAT));
                    break;

           case 'h':
                    en_wdg_pulse='1';
                    printk("-------------------------\n");
                    printk("en_wdg_pulse: Help Menu\n");
                    printk("-------------------------\n");
                    printk("1: Enable Watchdog Pulse\n");
                    printk("0: Disable Watchdog Pulse\n");
                    printk("h: Show this Help Menu\n");
                    break;
            default:
                    printk("Invaild Value!\n");
                    return -EFAULT;
        }
        return count;
    }
    return -EFAULT;
}

int en_wdg_pulse_single_open(struct inode *inode, struct file *file)
{
        return(single_open(file, en_wdg_pulse_read_proc, NULL));
}

static ssize_t en_wdg_pulse_single_write(struct file * file, const char __user * userbuf,
             size_t count, loff_t * off)
{
    return en_wdg_pulse_write_proc(file, userbuf,count, off);
}

static const struct file_operations en_wdg_pulse_proc_fops = {
    .open       = en_wdg_pulse_single_open,
    .write      = en_wdg_pulse_single_write,
    .read       = seq_read,
    .llseek     = seq_lseek,
    .release    = single_release,
};

static int __init rtl819x_gpio_peripheral_init(void)
{
	int err;

	rtl819x_gpio_peripheral.ngpio = BSP_GPIO_PIN_MAX;
	err = gpiochip_add(&rtl819x_gpio_peripheral);

	if (err)
		panic("cannot add rtl89x BSP_GPIO chip, error=%d", err);


	//Initial usb power (0:Enable 1:Disable)
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G0);
	rtl819x_gpio_direction_output(&rtl819x_gpio_peripheral, BSP_GPIO_PIN_G0, 0);
	rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G0, 0);

	proc_create_data("usb_power", 0, NULL, &usb_power_proc_fops, NULL);

    //Let HW RESET PIN Enable (0:Enable, 1:Disable)
    REG32(0xb8000008) = REG32(0xb8000008) &(~(1<<5));
    rtl819x_gpio_pin_enable(BSP_GPIO_PIN_B5);
    rtl819x_gpio_direction_input(&rtl819x_gpio_peripheral, BSP_GPIO_PIN_B5);
	printk("HW reset pin enabled\n");

    //Enable watchdog
    rtl819x_gpio_pin_enable(BSP_GPIO_PIN_F4);
    rtl819x_gpio_direction_output(&rtl819x_gpio_peripheral, BSP_GPIO_PIN_F4, 1);
    rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_F4, 0);
    
    //Initial watchdog pulse
    rtl819x_gpio_pin_enable(BSP_GPIO_PIN_E1);
    rtl819x_gpio_direction_output(&rtl819x_gpio_peripheral, BSP_GPIO_PIN_E1, 0);

    //Create pulse for external watchdog
    init_timer(&watchdog_timer);
    watchdog_timer.expires = jiffies + HZ;
    watchdog_timer.data = 0;
    watchdog_timer.function = &watchdog_function;
    mod_timer(&watchdog_timer, jiffies + HZ);

    proc_create_data("en_wdg_pulse", 0, NULL, &en_wdg_pulse_proc_fops,NULL);

	return err;
}
device_initcall(rtl819x_gpio_peripheral_init);

int gpio_to_irq(unsigned gpio)
{
    /* FIXME */
    return -EINVAL;
}
EXPORT_SYMBOL(gpio_to_irq);

int irq_to_gpio(unsigned irq)
{
    /* FIXME */
    return -EINVAL;
}
EXPORT_SYMBOL(irq_to_gpio);  

static unsigned int rtl819x_gpio_mux(u32 pin, u32 *value, u32 *address )
{
	unsigned int mask = 0;

	switch(pin) {
        case BSP_UART1_PIN:   
		mask = ((0xf<<3)|(7<<7)|(7<<10)|(0xf<<13));
		*value =  (0x7<<3)|(7<<7)|(7<<10)|(0x8<<13);
		*address=0xB800010C; /*SYS_PIN_MUX_SEL4*/
		break;

        case BSP_UART2_PIN:   
		mask = (0x1<<22)|(3<<23)|(3<<25);
		*value =  (0x0<<22)|(0<<23)|(0<<25);
		*address=0xB8000104;  /*SYS_PIN_MUX_SEL2*/
		break;

		case BSP_GPIO_PIN_E7: //SW reset button
		mask = 0x7<<27;
		*value =  0x3<<27;
		*address=0xB8000108;
		break;
		case BSP_GPIO_PIN_H5:	//RESET BTN
		mask = 0x3<<23;
		*value =  0x3<<23;
		*address=0xB800010C;
		break;
		case BSP_GPIO_PIN_H6:	//WPS BTN
		mask = 0x3<<0;
		*value =  0x3<<0;
		*address=0xB8000110;
		break;
		case BSP_GPIO_PIN_E2:	//WPS LED
		mask = 0x7<<15;
		*value =  0x3<<15;
		*address=0xB8000108;
		break;
		case BSP_GPIO_PIN_F3:	//Port3 LED
		mask = 0x3<<21;
		*value =  0x3<<21;
		*address=0xB8000108;
		break;
		case BSP_GPIO_PIN_F5:	//WPS button
		mask = 0x3<<0;
		*value =  0x3<<0;
		*address=0xB800010C;
		break;
		case BSP_GPIO_PIN_G0:	//USB power
		mask = 0x7<<7;
		*value =  0x3<<7;
		*address=0xB800010C;
		break;
		case BSP_GPIO_PIN_B5:	//HW reset enable
		mask = 0x7<<9;
		*value =  0x3<<9;
		*address=0xB8000108;
		break;
		case BSP_GPIO_PIN_F4:	//watchdog enable
		case BSP_GPIO_PIN_E1:	//watchdog
		mask = 0x7<<15;
		*value =  0x3<<15;
		*address=0xB8000108;
		break;
	default:
		break;
	}

	return mask;
}


