#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/version.h>
#include <linux/sched.h>
#include <linux/pid.h>
#include <linux/spinlock.h>
#include <linux/kmod.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include "bspchip.h"

//#define ACN_GPIO_DEBUG

#ifdef ACN_GPIO_DEBUG
#   define DPRINTK(fmt, args...) printk("[%s]:"fmt "\n", __FUNCTION__, ##args)
#else
#   define DPRINTK(fmt, args...)
#endif


#define PROBE_TIME      5
#define PROBE_NULL      0
#define PROBE_ACTIVE    1
#define PROBE_RESET     2
#define PROBE_RELOAD    3

static struct timer_list watchdog_timer;
static struct timer_list probe_timer;
static unsigned int    probe_counter;
static unsigned int    probe_state;

static unsigned int rtl819x_gpio_mux(u32 pin);
static DEFINE_SPINLOCK(rtl819x_gpio_lock);
static char default_flag='0';
static char usb_power='0';
static char pse_power='0';
static char wifi_led_interact_time='5';

typedef enum {
	RSSI_LED_OFF_ALL	=	'0',
	RSSI_LED_ON_1   	=	'1',
	RSSI_LED_ON_12		=	'2',
	RSSI_LED_ON_123		=	'3',
	RSSI_LED_ON_1234	=	'4',
} rssi_led_state_e;

static char rssi_led='4';
static char rssi_led_low_active='1';

typedef enum {
	WIFI_LED_OFF_ALL	=	'0',
	WIFI_LED_ON_radio_0	=	'1', //5G
	WIFI_LED_ON_radio_1	=	'2', //2.4G
	WIFI_LED_ON_ALL		=	'3',
} wifi_led_state_e;

static char wifi_led='0';
static char wifi_led_low_active='1';


static void rtl819x_gpio_pin_enable(u32 pin)
{
	unsigned long flags;
	unsigned int mask  = 0;
	unsigned int mux =0;

	if (pin >= BSP_GPIO_PIN_MAX)
		return;

	spin_lock_irqsave(&rtl819x_gpio_lock, flags);

	/* pin MUX1 */
	mask = rtl819x_gpio_mux(pin);
	mux  = __raw_readl((void __iomem*) BSP_PINMUX_SEL_REG(pin));
	if (mask != 0 && (mux & mask) == 0)
		__raw_writel( mux | mask, (void __iomem*) BSP_PINMUX_SEL_REG(pin));
	else if (pin == BSP_GPIO_PIN_B4){ //B4:default value of PIN_MUX_SEL_2 is 0x10
		__raw_writel( mux | mask, (void __iomem*) BSP_PINMUX_SEL_REG(pin));
	}

	/* 0 as BSP_GPIO pin */
	__raw_writel(__raw_readl((void __iomem*)BSP_GPIO_CNR_REG(pin)) & ~(1<<BSP_GPIO_BIT(pin)),
					(void __iomem*)BSP_GPIO_CNR_REG(pin));

	spin_unlock_irqrestore(&rtl819x_gpio_lock, flags);
}

static int rtl819x_gpio_direction_output(unsigned pin)
{
	unsigned long flags;

	if (pin >= BSP_GPIO_PIN_MAX)
		return -1;

	spin_lock_irqsave(&rtl819x_gpio_lock, flags);

	__raw_writel(__raw_readl((void __iomem*)BSP_GPIO_DIR_REG(pin)) | (1 << BSP_GPIO_BIT(pin)),
					(void __iomem*)BSP_GPIO_DIR_REG(pin));

	spin_unlock_irqrestore(&rtl819x_gpio_lock, flags);

	return 0;
}



static int rtl819x_gpio_pin_set_value(unsigned pin, int value)
{
	unsigned int data;

	data = __raw_readl((void __iomem*)BSP_GPIO_DAT_REG(pin));

	if (value == 0)
		data &= ~(1 << BSP_GPIO_BIT(pin));
	else
		data |= (1 << BSP_GPIO_BIT(pin));

	__raw_writel(data, (void __iomem*)BSP_GPIO_DAT_REG(pin));

	return 0;
}


static int rtl819x_gpio_pin_get_value(unsigned gpio)
{
	unsigned int data;

	data = (__raw_readl((void __iomem*)BSP_GPIO_DAT_REG(gpio)) >> BSP_GPIO_BIT(gpio) ) & 1;

	return data;
}

// load_default
static int load_default_read_proc(struct seq_file *m, void *v)
{

	seq_printf(m, "%c\n", default_flag);
	return 0;
}

int load_default_single_open(struct inode *inode, struct file *file)
{
	return(single_open(file, load_default_read_proc, NULL));
}


static ssize_t load_default_write_proc(struct file *file, const char __user *buffer,
				size_t count, loff_t *pos)
{
	if (count < 2)
		return -EFAULT;
	if (buffer && !copy_from_user(&default_flag, buffer, 1)) {
		return count;
	}
	return -EFAULT;
}

static ssize_t load_default_single_write(struct file * file, const char __user * userbuf,
		size_t count, loff_t * off)
{
	return load_default_write_proc(file, userbuf,count, off);
}

// usb_power
static int usb_power_read_proc(struct seq_file *m, void *v)
{
	seq_printf(m, "%c\n", usb_power);
	return 0;
}

int usb_power_single_open(struct inode *inode, struct file *file)
{
	return(single_open(file, usb_power_read_proc, NULL));
}

static ssize_t usb_power_write_proc(struct file *file, const char *buffer,
				size_t count, loff_t *ppos)
{
	if (count < 2)
		return -EFAULT;
	if (buffer && !copy_from_user(&usb_power, buffer, 1)) {
		if ( usb_power == '0')
			rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_G6, 1);
		else if (usb_power == '1')
			rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_G6, 0);
		else{
			printk("Invalid value!\n");
			return -EFAULT;
		}
		return count;
	}
	return -EFAULT;
}

static ssize_t usb_power_single_write(struct file * file, const char __user * userbuf,
		size_t count, loff_t * off)
{
	return usb_power_write_proc(file, userbuf,count, off);
}


// pse_power
static int pse_power_read_proc(struct seq_file *m, void *v)
{
	seq_printf(m, "%c\n", pse_power);
	return 0;
}

int pse_power_single_open(struct inode *inode, struct file *file)
{
	return(single_open(file, pse_power_read_proc, NULL));
}

static ssize_t pse_power_write_proc(struct file *file, const char *buffer,
				size_t count, loff_t *ppos)
{
	if (count < 2)
		return -EFAULT;
	if (buffer && !copy_from_user(&pse_power, buffer, 1)) {
		if ( pse_power == '0')
			rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_G7, 1);
		else if (pse_power == '1')
			rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_G7, 0);
		else {
			printk("Invalid value!\n");
			return -EFAULT;
		}
		return count;
	}
	return -EFAULT;
}

static ssize_t pse_power_single_write(struct file * file, const char __user * userbuf,
		size_t count, loff_t * off)
{
	return pse_power_write_proc(file, userbuf,count, off);
}


// wifi_led_low_active 
static int wifi_led_low_active_read_proc(struct seq_file *m, void *v)
{
	seq_printf(m, "%c\n", wifi_led_low_active);
	return 0;
}

int wifi_led_low_active_single_open(struct inode *inode, struct file *file)
{
	return(single_open(file, wifi_led_low_active_read_proc, NULL));
}

static ssize_t wifi_led_low_active_write_proc(struct file *file, const char *buffer,
				size_t count, loff_t *ppos)
{
	if (count < 2)
		return -EFAULT;
	if (buffer && !copy_from_user(&wifi_led_low_active, buffer, 1)) {
		return count;
	}
	return -EFAULT;
}

static ssize_t wifi_led_low_active_single_write(struct file * file, const char __user * userbuf,
		size_t count, loff_t * off)
{
	return wifi_led_low_active_write_proc(file, userbuf,count, off);
}

// wifi_led_interact_time
static int wifi_led_interact_time_read_proc(struct seq_file *m, void *v)
{
	seq_printf(m, "%c\n", wifi_led_interact_time);
	return 0;
}

int wifi_led_interact_time_single_open(struct inode *inode, struct file *file)
{
	return(single_open(file, wifi_led_interact_time_read_proc, NULL));
}

static ssize_t wifi_led_interact_time_write_proc(struct file *file, const char *buffer,
				size_t count, loff_t *ppos)
{
	if (count < 2)
		return -EFAULT;
	if (buffer && !copy_from_user(&wifi_led_interact_time, buffer, 1)) {
		return count;
	}
	return -EFAULT;
}

static ssize_t wifi_led_interact_time_single_write(struct file * file, const char __user * userbuf,
		size_t count, loff_t * off)
{
	return wifi_led_interact_time_write_proc(file, userbuf,count, off);
}

// wifi_led
static void wifi_led_off(unsigned radio)
{
	unsigned pin;

	switch (radio){
		case 0:
			pin = BSP_GPIO_PIN_G5;
			break;
		case 1:
			pin = BSP_GPIO_PIN_G4;
			break;
		default:
			printk("Invalid Value!\n");
			return;
	}
	if (wifi_led_low_active == '1'){
		rtl819x_gpio_pin_set_value(pin,1);
	}else{
		rtl819x_gpio_pin_set_value(pin,0);
	}

}

static void wifi_led_on(unsigned radio)
{
	unsigned pin;

	switch (radio){
		case 0:
			pin = BSP_GPIO_PIN_G5;
			break;
		case 1:
			pin = BSP_GPIO_PIN_G4;
			break;
		default:
			printk("Invalid Value!\n");
			return;
	}
	if (wifi_led_low_active == '1'){
		rtl819x_gpio_pin_set_value(pin,0);
	}else{
		rtl819x_gpio_pin_set_value(pin,1);
	}
}

static int wifi_led_read_proc(struct seq_file *m, void *v)
{
	seq_printf(m, "%c\n", wifi_led);
	return 0;
}

int wifi_led_single_open(struct inode *inode, struct file *file)
{
	return(single_open(file, wifi_led_read_proc, NULL));
}

static ssize_t wifi_led_write_proc(struct file *file, const char *buffer,
				size_t count, loff_t *ppos)
{
	if (count < 2)
		return -EFAULT;
	if (buffer && !copy_from_user(&wifi_led, buffer, 1))
	{
		switch(wifi_led)
		{
			case WIFI_LED_OFF_ALL:
				wifi_led_off(0);
				wifi_led_off(1);
				break;
			case WIFI_LED_ON_radio_0: //5G
				wifi_led_on(0);
				wifi_led_off(1);
				break;
			case WIFI_LED_ON_radio_1: //2.4G
				wifi_led_off(0);
				wifi_led_on(1);
				break;
			case WIFI_LED_ON_ALL:
				wifi_led_on(0);
				wifi_led_on(1);
				break;
			default:
				printk("Invalid Value!\n");
				return -EFAULT;
		}
		DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_PINMUX_SEL_REG(BSP_GPIO_PIN_G4), __raw_readl((void __iomem*)BSP_PINMUX_SEL_REG(BSP_GPIO_PIN_G4)));
		DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_CNR_REG(BSP_GPIO_PIN_G4), __raw_readl((void __iomem*)BSP_GPIO_CNR_REG(BSP_GPIO_PIN_G4)));
		DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_DIR_REG(BSP_GPIO_PIN_G4), __raw_readl((void __iomem*)BSP_GPIO_DIR_REG(BSP_GPIO_PIN_G4)));
		DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_DAT_REG(BSP_GPIO_PIN_G4), __raw_readl((void __iomem*)BSP_GPIO_DAT_REG(BSP_GPIO_PIN_G4)));		
		return count;
	}
	return -EFAULT;
}


static ssize_t wifi_led_single_write(struct file * file, const char __user * userbuf,
		size_t count, loff_t * off)
{
	return wifi_led_write_proc(file, userbuf,count, off);
}

// rssi_led_low_active
static int rssi_led_low_active_read_proc(struct seq_file *m, void *v)
{
	seq_printf(m, "%c\n", rssi_led_low_active);
	return 0;
}

int rssi_led_low_active_single_open(struct inode *inode, struct file *file)
{
	return(single_open(file, rssi_led_low_active_read_proc, NULL));
}

static ssize_t rssi_led_low_active_write_proc(struct file *file, const char *buffer,
				size_t count, loff_t *ppos)
{
	if (count < 2)
		return -EFAULT;
	if (buffer && !copy_from_user(&rssi_led_low_active, buffer, 1)) {
		return count;
	}
	return -EFAULT;
}

static ssize_t rssi_led_low_active_single_write(struct file * file, const char __user * userbuf,
		size_t count, loff_t * off)
{
	return rssi_led_low_active_write_proc(file, userbuf,count, off);
}


// rssi_led
static void rssi_led_off(unsigned led)
{
	unsigned pin;

	switch (led){
		case 1:
			pin = BSP_GPIO_PIN_G0;
			break;
		case 2:
			pin = BSP_GPIO_PIN_G1;
			break;
		case 3:
			pin = BSP_GPIO_PIN_G2;
			break;
		case 4:
			pin = BSP_GPIO_PIN_G3;
			break;
		default:
			printk("Invalid Value!\n");
			return;
	}
	if(rssi_led_low_active == '1'){
		rtl819x_gpio_pin_set_value(pin,1);
	}else{
		rtl819x_gpio_pin_set_value(pin,0);
	}
}

static void rssi_led_on(unsigned led)
{
	unsigned pin;

	switch (led){
		case 1:
			pin = BSP_GPIO_PIN_G0;
			break;
		case 2:
			pin = BSP_GPIO_PIN_G1;
			break;
		case 3:
			pin = BSP_GPIO_PIN_G2;
			break;
		case 4:
			pin = BSP_GPIO_PIN_G3;
			break;
		default:
			printk("Invalid Value!\n");
			return;
	}
	if(rssi_led_low_active == '1'){
		rtl819x_gpio_pin_set_value(pin,0);
	}else{
		rtl819x_gpio_pin_set_value(pin,1);
	}
}


static int rssi_led_read_proc(struct seq_file *m, void *v)
{
	seq_printf(m, "%c\n", rssi_led);
	return 0;
}

int rssi_led_single_open(struct inode *inode, struct file *file)
{
	return(single_open(file, rssi_led_read_proc, NULL));
}

static ssize_t rssi_led_write_proc(struct file *file, const char *buffer,
				size_t count, loff_t *ppos)
{
	if (count < 2)
		return -EFAULT;
	if (buffer && !copy_from_user(&rssi_led, buffer, 1))
	{
		switch(rssi_led)
		{
			case RSSI_LED_OFF_ALL:
				rssi_led_off(1);
				rssi_led_off(2);
				rssi_led_off(3);
				rssi_led_off(4);
				break;
			case RSSI_LED_ON_1:
				rssi_led_on(1);
				rssi_led_off(2);
				rssi_led_off(3);
				rssi_led_off(4);
				break;
			case RSSI_LED_ON_12:
				rssi_led_on(1);
				rssi_led_on(2);
				rssi_led_off(3);
				rssi_led_off(4);
				break;
			case RSSI_LED_ON_123:
				rssi_led_on(1);
				rssi_led_on(2);
				rssi_led_on(3);
				rssi_led_off(4);
				break;
			case RSSI_LED_ON_1234:
				rssi_led_on(1);
				rssi_led_on(2);
				rssi_led_on(3);
				rssi_led_on(4);
				break;
			default:
				printk("Invalid Value!\n");
				return -EFAULT;
		}
		DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_PINMUX_SEL_REG(BSP_GPIO_PIN_G0), __raw_readl((void __iomem*)BSP_PINMUX_SEL_REG(BSP_GPIO_PIN_G0)));
		DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_CNR_REG(BSP_GPIO_PIN_G0), __raw_readl((void __iomem*)BSP_GPIO_CNR_REG(BSP_GPIO_PIN_G0)));
		DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_DIR_REG(BSP_GPIO_PIN_G0), __raw_readl((void __iomem*)BSP_GPIO_DIR_REG(BSP_GPIO_PIN_G0)));
		DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_DAT_REG(BSP_GPIO_PIN_G0), __raw_readl((void __iomem*)BSP_GPIO_DAT_REG(BSP_GPIO_PIN_G0)));
		return count;
	}
	return -EFAULT;
}

static ssize_t rssi_led_single_write(struct file * file, const char __user * userbuf,
		size_t count, loff_t * off)
{
	return rssi_led_write_proc(file, userbuf,count, off);
}


static void reset_reboot_probe_check(unsigned long data)
{
	unsigned int pressed=1;


	if (rtl819x_gpio_pin_get_value(BSP_GPIO_PIN_A5))
	{
		pressed = 0;
	}
	else
	{
		printk("Key pressed %d!\n", probe_counter+1);
	}


	if (probe_state == PROBE_NULL)
	{
		if (pressed)
		{
			probe_state = PROBE_ACTIVE;
			probe_counter++;
		}
		else
			probe_counter = 0;
	}
	else if (probe_state == PROBE_ACTIVE)
	{
		if (pressed)
		{
			probe_counter++;

			if ((probe_counter >=2 ) && (probe_counter <=PROBE_TIME))
			{
				DPRINTK("2-5\n");
			}
			else if (probe_counter >= PROBE_TIME)
			{
				DPRINTK(">5 \n");
			}
		}
		else
		{
			if (probe_counter < 1)
			{
				probe_state = PROBE_NULL;
				probe_counter = 0;
				DPRINTK("<2 \n");
			}
			else if (probe_counter >= PROBE_TIME)
			{
				DPRINTK("default_flag\n");
				pr_info("Push button more than 5 ----> Reload Default\n");
				default_flag='1';
				probe_counter = 0;
				probe_state = PROBE_NULL;
			}
			else
			{
				DPRINTK("2-5 reboot system\n");
				pr_info("Push button less than 5 ----> Restart System\n");
				//pid = get_pid(find_vpid(1));
				//kill_pid(pid,SIGTERM,1);
				default_flag='2';
				probe_counter = 0;
				probe_state = PROBE_NULL;
			}
		}
	}

	mod_timer(&probe_timer, jiffies + HZ);
}


static void watchdog_function(unsigned long data)
{
	if (watchdog_timer.data==0)
	{
		rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_C0, 1);
		watchdog_timer.data=1;
	} else {
		rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_C0, 0);
		watchdog_timer.data=0;
	}

	DPRINTK("Check watchdog");
	DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_PINMUX_SEL_REG(BSP_GPIO_PIN_B4), __raw_readl((void __iomem*)BSP_PINMUX_SEL_REG(BSP_GPIO_PIN_B4)));
	DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_CNR_REG(BSP_GPIO_PIN_B4), __raw_readl((void __iomem*)BSP_GPIO_CNR_REG(BSP_GPIO_PIN_B4)));
	DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_DIR_REG(BSP_GPIO_PIN_B4), __raw_readl((void __iomem*)BSP_GPIO_DIR_REG(BSP_GPIO_PIN_B4)));
	DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_DAT_REG(BSP_GPIO_PIN_B4), __raw_readl((void __iomem*)BSP_GPIO_DAT_REG(BSP_GPIO_PIN_B4)));

	#if 1  /*wdt strobe time monitor, stoneshih*/
	{	
		#define WDT_STROBE_WARNING_TIME 3*1000*400  //3 second
		static unsigned prev_time = 1;
		unsigned cur_time;
		cur_time = *(unsigned*)0xb800310c;//up count in 2.5us unit
		if (prev_time != 1) {
			unsigned diff = cur_time - prev_time;
			if (diff >= WDT_STROBE_WARNING_TIME)
				printk("\nstrobe wdt in %d ms\n",(cur_time - prev_time)/400);
		}
		else
			*(volatile unsigned*)(0xB8003110) = 0xF0000000;//enable timer1	
		prev_time = cur_time;
	}
	#endif

	mod_timer(&watchdog_timer, jiffies + 100);
}


static const struct file_operations load_default_fops = {
	.owner = THIS_MODULE,
	.open = load_default_single_open,
	.write = load_default_single_write,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release,
};

static const struct file_operations rssi_led_fops = {
	.owner = THIS_MODULE,
	.open = rssi_led_single_open,
	.write = rssi_led_single_write,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release
};

static const struct file_operations rssi_led_low_active_fops = {
	.owner = THIS_MODULE,
	.open = rssi_led_low_active_single_open,
	.write = rssi_led_low_active_single_write,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release
};

static const struct file_operations wifi_led_fops = {
	.owner = THIS_MODULE,
	.open = wifi_led_single_open,
	.write = wifi_led_single_write,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release
};

static const struct file_operations wifi_led_interact_time_fops = {
	.owner = THIS_MODULE,
	.open = wifi_led_interact_time_single_open,
	.write = wifi_led_interact_time_single_write,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release
};

static const struct file_operations wifi_led_low_active_fops = {
	.owner = THIS_MODULE,
	.open = wifi_led_low_active_single_open,
	.write = wifi_led_low_active_single_write,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release
};

static const struct file_operations usb_power_fops = {
	.owner = THIS_MODULE,
	.open = usb_power_single_open,
	.write = usb_power_single_write,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release
};

static const struct file_operations pse_power_fops = {
	.owner = THIS_MODULE,
	.open = pse_power_single_open,
	.write = pse_power_single_write,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release
};


static int __init acn_gpio_init(void)
{

	pr_info("==== ACN GPIO Module Init. ====\n");

	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_B4);
	rtl819x_gpio_direction_output(BSP_GPIO_PIN_B4);
	rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_B4, 1);

	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_C0);
	rtl819x_gpio_direction_output(BSP_GPIO_PIN_C0);

	//Create pulse for external watchdog
	init_timer(&watchdog_timer);
	watchdog_timer.expires = jiffies + 100;
	watchdog_timer.data = 0;
	watchdog_timer.function = &watchdog_function;
	mod_timer(&watchdog_timer, jiffies + 100);


	//Reset and Reboot Button
	init_timer(&probe_timer);
	probe_counter = 0;
	probe_state = PROBE_NULL;
	probe_timer.expires = jiffies + HZ;
	probe_timer.data = (unsigned long)NULL;
	probe_timer.function = &reset_reboot_probe_check;
	mod_timer(&probe_timer, jiffies + HZ);

	proc_create("load_default", 0, NULL, &load_default_fops);

	//RSSI LED Init value
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G0);
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G1);
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G2);
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G3);
	rtl819x_gpio_direction_output(BSP_GPIO_PIN_G0);
	rtl819x_gpio_direction_output(BSP_GPIO_PIN_G1);
	rtl819x_gpio_direction_output(BSP_GPIO_PIN_G2);
	rtl819x_gpio_direction_output(BSP_GPIO_PIN_G3);
	rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_G0,0); //0:on 1:off
	rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_G1,0);
	rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_G2,0);
	rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_G3,0);

	proc_create("rssi_led", 0, NULL, &rssi_led_fops);
	proc_create("rssi_led_low_active", 0, NULL, &rssi_led_low_active_fops);

	//WIFI LED Init value
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G4);
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G5);
	rtl819x_gpio_direction_output(BSP_GPIO_PIN_G4);
	rtl819x_gpio_direction_output(BSP_GPIO_PIN_G5);
	rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_G4,1); //0:on 1:off
	rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_G5,1);

	proc_create("wifi_led", 0, NULL, &wifi_led_fops);
	proc_create("wifi_led_interact_time", 0, NULL, &wifi_led_interact_time_fops);
	proc_create("wifi_led_low_active", 0, NULL, &wifi_led_low_active_fops);

	//USB POWER Init value
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G6);
	rtl819x_gpio_direction_output(BSP_GPIO_PIN_G6);
	rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_G6,1); //0:on 1:off

	proc_create("usb_power", 0, NULL, &usb_power_fops);

	//PSE POWER Init value
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G7);
	rtl819x_gpio_direction_output(BSP_GPIO_PIN_G7);
	rtl819x_gpio_pin_set_value(BSP_GPIO_PIN_G7,1); //0:on 1:off

	proc_create("pse_power", 0, NULL, &pse_power_fops);

	DPRINTK("Init");
	DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_PINMUX_SEL_REG(BSP_GPIO_PIN_B4), __raw_readl((void __iomem*)BSP_PINMUX_SEL_REG(BSP_GPIO_PIN_B4)));
	DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_CNR_REG(BSP_GPIO_PIN_B4), __raw_readl((void __iomem*)BSP_GPIO_CNR_REG(BSP_GPIO_PIN_B4)));
	DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_DIR_REG(BSP_GPIO_PIN_B4), __raw_readl((void __iomem*)BSP_GPIO_DIR_REG(BSP_GPIO_PIN_B4)));
	DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_DAT_REG(BSP_GPIO_PIN_B4), __raw_readl((void __iomem*)BSP_GPIO_DAT_REG(BSP_GPIO_PIN_B4)));
	DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_PINMUX_SEL_REG(BSP_GPIO_PIN_G6), __raw_readl((void __iomem*)BSP_PINMUX_SEL_REG(BSP_GPIO_PIN_G6)));
	DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_CNR_REG(BSP_GPIO_PIN_G6), __raw_readl((void __iomem*)BSP_GPIO_CNR_REG(BSP_GPIO_PIN_G6)));
	DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_DIR_REG(BSP_GPIO_PIN_G6), __raw_readl((void __iomem*)BSP_GPIO_DIR_REG(BSP_GPIO_PIN_G6)));
	DPRINTK("read reg:0x%8X data:0x%8X", (void __iomem*)BSP_GPIO_DAT_REG(BSP_GPIO_PIN_G6), __raw_readl((void __iomem*)BSP_GPIO_DAT_REG(BSP_GPIO_PIN_G6)));
	return 0;
}

device_initcall(acn_gpio_init);


static unsigned int rtl819x_gpio_mux(u32 pin)
{
	unsigned int mask = 0;

	switch(pin) {
		/*
		 * PIN SEL 1
		 */
		case BSP_GPIO_PIN_F5:
		case BSP_GPIO_PIN_F6:
		/* 0xB800-0040 bit 4:3 */
		mask = 0x00000018;
		break;

		case BSP_GPIO_PIN_B1:
		/* 0xB800-0040 bit 6 */
		mask = 0x00000040;
		break;

		case BSP_GPIO_PIN_G0:
		case BSP_GPIO_PIN_G1:
		case BSP_GPIO_PIN_G2:
		case BSP_GPIO_PIN_G3:
		case BSP_GPIO_PIN_G4:
		case BSP_GPIO_PIN_G5:
		case BSP_GPIO_PIN_G6:
		case BSP_GPIO_PIN_G7:
		/* 0xB800-0040 bit 11:10 */
		mask = 0x00000C00;
		break;

		case BSP_GPIO_PIN_A1:
		/* 0xB800-0040 bit 13:12 */
		mask = 0x00003000;
		break;

		case BSP_GPIO_PIN_A0:
		/* 0xB800-0040 bit 17:16 */
		mask = 0x00030000;
		break;

		case BSP_GPIO_PIN_C6:
		/* 0xB800-0040 bit 19:18 */
		mask = 0x000C0000;
		break;

		case BSP_GPIO_PIN_C1:
		/* 0xB800-0040 bit 21:20 */
		mask = 0x00300000;
		break;

		case BSP_GPIO_PIN_C2:
		/* 0xB800-0040 bit 23:22 */
		mask = 0x00C00000;
		break;

		case BSP_GPIO_PIN_C3:
		/* 0xB800-0040 bit 25:24 */
		mask = 0x03000000;
		break;

		case BSP_GPIO_PIN_C4:
		/* 0xB800-0040 bit 27:26 */
		mask = 0x0C000000;
		break;

		case BSP_GPIO_PIN_C5:
		/* 0xB800-0040 bit 29:28 */
		mask = 0x30000000;
		break;


		/*
		 * PIN SEL 2
		 */
		case BSP_GPIO_PIN_B2:
		/* 0xB800-0044 bit 1:0 */
		mask = 0x00000003;
		break;

		case BSP_GPIO_PIN_B3:
		/* 0xB800-0044 bit 4:3 */
		mask = 0x00000018;
		break;

		case BSP_GPIO_PIN_B4:
		/* 0xB800-0044 bit 7:6 */
		mask = 0x000000C0;
		break;

		case BSP_GPIO_PIN_B5:
		/* 0xB800-0044 bit 10:9 */
		mask = 0x00000600;
		break;

		case BSP_GPIO_PIN_B6:
		/* 0xB800-0044 bit 13:12 */
		mask = 0x00003000;
		break;

		case BSP_GPIO_PIN_B7:
		/* 0xB800-0044 bit 17:15 */
		mask = 0x00020000;
		break;

		case BSP_GPIO_PIN_C0:
		/* 0xB800-0044 bit 20:18 */
		mask = 0x00100000;
		break;

		default:
		break;
	}

	return mask;
}

