/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2007 Aurelien Jarno <aurelien@aurel32.net>
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/spinlock.h>
#include <linux/gpio.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include "bspchip.h"

static struct timer_list watchdog_timer;
static char en_wdg_pulse='1';

static unsigned int rtl819x_gpio_mux(u32 pin, u32 *value, u32 *address );

static DEFINE_SPINLOCK(rtl819x_gpio_lock);

static int __rtl819x_gpio_get_value(unsigned gpio)
{
	unsigned int data;

	data = (__raw_readl((void __iomem*)BSP_GPIO_DAT_REG(gpio)) >> BSP_GPIO_BIT(gpio) ) & 1;

	return data;
}

static int rtl819x_gpio_get_value(struct gpio_chip *chip, unsigned pin)
{
	if (pin >= BSP_GPIO_PIN_MAX)
		return -1;

	return __rtl819x_gpio_get_value(pin);
}

static int __rtl819x_gpio_set_value(unsigned pin, int value)
{
	unsigned int data;

	data = __raw_readl((void __iomem*)BSP_GPIO_DAT_REG(pin));

	#if 0
	printk("%s(%d)xxx 0x%08x\n",__FUNCTION__,__LINE__,
		__raw_readl((void __iomem*)PINMUX_SEL_REG(_ata_rtl8972d_v1xx_leds_gpio[i].gpio)));
	printk("%s(%d)xxx 0x%08x\n",__FUNCTION__,__LINE__,
		__raw_readl((void __iomem*)BSP_GPIO_CNR_REG(_ata_rtl8972d_v1xx_leds_gpio[i].gpio)));
	printk("%s(%d)xxx 0x%08x\n",__FUNCTION__,__LINE__,
		__raw_readl((void __iomem*)BSP_GPIO_DIR_REG(_ata_rtl8972d_v1xx_leds_gpio[i].gpio)));

	printk("%s(%d)xxx pin=%d, val=%d\n",__FUNCTION__,__LINE__,pin,value);
	printk("%s(%d)xxx 0x%08x\n",__FUNCTION__,__LINE__,REG32(BSP_GPIO_DAT_REG(pin)));
	#endif

	if (value == 0)
		data &= ~(1 << BSP_GPIO_BIT(pin));
	else
		data |= (1 << BSP_GPIO_BIT(pin));

	//printk("%s(%d)yyy 0x%08x\n",__FUNCTION__,__LINE__,data);

	__raw_writel(data, (void __iomem*)BSP_GPIO_DAT_REG(pin));

	//printk("%s(%d)xxx 0x%08x\n",__FUNCTION__,__LINE__,
	//	__raw_readl((void __iomem*)BSP_GPIO_DAT_REG(pin)));

	return 0;
}

static void rtl819x_gpio_set_value(struct gpio_chip *chip,
				  unsigned pin, int value)
{
	if (pin >= BSP_GPIO_PIN_MAX)
		return;

	__rtl819x_gpio_set_value(pin, value);
}

static int rtl819x_gpio_direction_input(struct gpio_chip *chip,
				       unsigned pin)
{
	unsigned long flags;
	if (pin >= BSP_GPIO_PIN_MAX)
		return -1;

	spin_lock_irqsave(&rtl819x_gpio_lock, flags);

	/* 0 : input */
	__raw_writel(__raw_readl((void __iomem*)BSP_GPIO_DIR_REG(pin)) & ~(1 << BSP_GPIO_BIT(pin)), 
					(void __iomem*)BSP_GPIO_DIR_REG(pin));

	spin_unlock_irqrestore(&rtl819x_gpio_lock, flags);

	return 0;
}

static int rtl819x_gpio_direction_output(struct gpio_chip *chip,
					unsigned pin, int value)
{
	unsigned long flags;

	if (pin >= BSP_GPIO_PIN_MAX)
		return -1;

	spin_lock_irqsave(&rtl819x_gpio_lock, flags);

	__raw_writel(__raw_readl((void __iomem*)BSP_GPIO_DIR_REG(pin)) | (1 << BSP_GPIO_BIT(pin)), 
					(void __iomem*)BSP_GPIO_DIR_REG(pin) );

	
	spin_unlock_irqrestore(&rtl819x_gpio_lock, flags);
	return 0;
}


static struct gpio_chip rtl819x_gpio_peripheral = {

	.label				= "rtl819x_gpio",
	.get				= rtl819x_gpio_get_value,
	.set				= rtl819x_gpio_set_value,
	.direction_input	= rtl819x_gpio_direction_input,
	.direction_output	= rtl819x_gpio_direction_output,
	.base				= 0,
};

void rtl819x_gpio_pin_enable(u32 pin)
{
	unsigned long flags;
	unsigned int mask  = 0;
	unsigned int mux =0;
	unsigned int mux_reg;
	unsigned int val;
	


	if (pin >= TOTAL_PIN_MAX)
		return;

	spin_lock_irqsave(&rtl819x_gpio_lock, flags);

	/* pin MUX1 */
	mask = rtl819x_gpio_mux(pin,&val,&mux_reg);

	//mux  = __raw_readl((void __iomem*) BSP_PINMUX_SEL_REG(pin));
	mux  = __raw_readl((void __iomem*)mux_reg);

	//if (mask != 0 && (mux & mask) == 0)
	if (mask != 0)
	    __raw_writel( ((mux&(~mask)) | (val)), (void __iomem*)mux_reg);

	/* 0 as BSP_GPIO pin */
	if(pin < BSP_GPIO_PIN_MAX)
	{
		__raw_writel(__raw_readl((void __iomem*)BSP_GPIO_CNR_REG(pin)) & ~(1<<BSP_GPIO_BIT(pin)),
					(void __iomem*)BSP_GPIO_CNR_REG(pin));
	}

	spin_unlock_irqrestore(&rtl819x_gpio_lock, flags);
}

void rtl819x_gpio_pin_disable(u32 pin)
{
	unsigned long flags;

	spin_lock_irqsave(&rtl819x_gpio_lock, flags);

	/* 1 as peripheral pin */
	__raw_writel(__raw_readl((void __iomem*)BSP_GPIO_CNR_REG(pin)) | (1<<BSP_GPIO_BIT(pin)), 
					(void __iomem*)BSP_GPIO_CNR_REG(pin));

	spin_unlock_irqrestore(&rtl819x_gpio_lock, flags);
}

void rtl819x_gpio_pin_set_val(u32 pin, int val)
{
	__rtl819x_gpio_set_value(pin, val);
}

int rtl819x_gpio_pin_get_val(u32 pin)
{
	return __rtl819x_gpio_get_value(pin);
}

//--------------------------------------------------------
//  Watchdog Pulse
//--------------------------------------------------------
static void watchdog_function(unsigned long data)
{
	if(en_wdg_pulse == '1') {
		if (watchdog_timer.data==0)
		{
			rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_B3, 1);
			watchdog_timer.data=1;
		} else {
			rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_B3, 0);
			watchdog_timer.data=0;
		}
	}

	#if 1  /*wdt strobe time monitor, stoneshih*/
	{	
		#define WDT_STROBE_WARNING_TIME 3*1000*400  //3 second
		static unsigned prev_time = 1;
		unsigned cur_time;
		cur_time = *(unsigned*)0xb800310c;//up count in 2.5us unit
		if (prev_time != 1) {
			unsigned diff = cur_time - prev_time;
			if (diff >= WDT_STROBE_WARNING_TIME)
				printk("\nstrobe wdt in %d ms\n",(cur_time - prev_time)/400);
		}
		else
			*(volatile unsigned*)(0xB8003110) = 0xF0000000;//enable timer1	
		prev_time = cur_time;
	}
	#endif

//increase wdt reboot safety margin for spark when extremely heavy traffic is continuously keeping
#if 0
	mod_timer(&watchdog_timer, jiffies + 100);
#else
	mod_timer(&watchdog_timer, jiffies + 50); 
#endif
}


//--------------------------------------------------------
//  Enable Watchdog Pulse
//--------------------------------------------------------
static int en_wdg_pulse_read_proc(struct seq_file *s, void *v)
{
    seq_printf(s,"en_wdg_pulse: %c\n", en_wdg_pulse);
    return 0;
}

static int en_wdg_pulse_write_proc(struct file *file, const char *buffer,
                unsigned long count, void *data)
{
    if (count < 2)
        return -EFAULT;

	if (buffer && !copy_from_user(&en_wdg_pulse, buffer, 1)) {
		switch(en_wdg_pulse)
		{
			case '1':
			case '0':
					break;
			case 'h':
					en_wdg_pulse='1';
					printk("-------------------------\n");
					printk("en_wdg_pulse: Help Menu\n");
					printk("-------------------------\n");
					printk("1: Enable Watchdog Pulse\n");
					printk("0: Disable Watchdog Pulse\n");
					printk("h: Show this Help Menu\n");
					break;
			default:
					printk("Invaild Value!\n");
					return -EFAULT;
		}
		return count;
	}
	return -EFAULT;
}

int en_wdg_pulse_single_open(struct inode *inode, struct file *file)
{
        return(single_open(file, en_wdg_pulse_read_proc, NULL));
}

static ssize_t en_wdg_pulse_single_write(struct file * file, const char __user * userbuf,
             size_t count, loff_t * off)
{
    return en_wdg_pulse_write_proc(file, userbuf,count, off);
}

static const struct file_operations en_wdg_pulse_proc_fops = {
    .open       = en_wdg_pulse_single_open,
    .write      = en_wdg_pulse_single_write,
    .read       = seq_read,
    .llseek     = seq_lseek,
    .release    = single_release,
};


//--------------------------------------------------------
//  LED Check
//--------------------------------------------------------
static int led_check_read_proc(struct seq_file *s, void *v)
{
#if 0 //For debug
	seq_printf(s,"PIN_MUX_SEL_1: 0x%08x\n", REG32(BSP_PIN_MUX_SEL1));
	seq_printf(s,"PIN_MUX_SEL_2: 0x%08x\n", REG32(BSP_PIN_MUX_SEL2));
	seq_printf(s,"PIN_MUX_SEL_3: 0x%08x\n", REG32(0xb800004c));
	seq_printf(s,"BSP_PABCD_CNR: 0x%08x\n",  REG32(BSP_PABCD_CNR));
	seq_printf(s,"BSP_PABCD_DIR: 0x%08x\n",  REG32(BSP_PABCD_DIR));
	seq_printf(s,"BSP_PABCD_DAT: 0x%08x\n",  REG32(BSP_PABCD_DAT));
	seq_printf(s,"BSP_PEFGH_CNR: 0x%08x\n",  REG32(BSP_PEFGH_CNR));
	seq_printf(s,"BSP_PEFGH_DIR: 0x%08x\n",  REG32(BSP_PEFGH_DIR));
	seq_printf(s,"BSP_PEFGH_DAT: 0x%08x\n\n",REG32(BSP_PEFGH_DAT));
#endif
	seq_printf(s,"--------------------------------------\n");
	seq_printf(s,"(H):High active  (L):Low active\n");
	seq_printf(s,"--------------------------------------\n");
	seq_printf(s,"WPS   BTN  A3: %d (L)\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_A3));
	seq_printf(s,"Reset BTN  A4: %d (L)\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_A4));
	seq_printf(s,"USB0 Power G7: %d (L)\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G7));
	seq_printf(s,"USB1 Power G5: %d (L)\n\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G5));

	seq_printf(s,"wlan0      G3: %d (H)\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G3));
	seq_printf(s,"Wlan1      G2: %d (H)\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G2));
	seq_printf(s,"rssi0      G1: %d (H)\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G1));

	seq_printf(s,"wdg enable G4: %d (H)\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G4));
	seq_printf(s,"en_wdg_pulse : %c\n", en_wdg_pulse);
	seq_printf(s,"wdg pluse  B3: %d\n\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_B3));

	return 0;
}

static int led_check_write_proc(struct file *file, const char *buffer,
                unsigned long count, void *data)
{
	char led_check;

	if (count < 2)
		return -EFAULT;

	if (buffer && !copy_from_user(&led_check, buffer, 1)) {
		switch(led_check)
		{
			case 'i':
				rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G3); // wlan0
				rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G2); // wlan1
				rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G1); // rssi0

				rtl819x_gpio_direction_output(&rtl819x_gpio_peripheral, BSP_GPIO_PIN_G3, 1);
				rtl819x_gpio_direction_output(&rtl819x_gpio_peripheral, BSP_GPIO_PIN_G2, 1);
				rtl819x_gpio_direction_output(&rtl819x_gpio_peripheral, BSP_GPIO_PIN_G1, 1);

				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G3, 0);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G2, 0);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G1, 0);
				break;
			case '0':
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G3, 0);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G2, 0);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G1, 0);
				break;
			case '1':
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G3, 1);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G2, 1);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G1, 1);
				break;
			case '3':
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G3, 1);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G2, 1);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G1, 1);
				break;
			case '4':
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G3, 0);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G2, 0);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G1, 0);
				break;
			case 'a':
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G3, 1);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G2, 0);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G1, 0);
				break;
			case 'b':
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G3, 0);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G2, 1);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G1, 0);
				break;
			case 'c':
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G3, 0);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G2, 0);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G1, 1);
				break;
			case 'h':
				printk("-------------------------\n");
				printk("led_check: Help Menu\n");
				printk("-------------------------\n");
				printk("i: Initial to GPIO\n");
				printk("1: Write value 1 to all GPIO Pins\n");
				printk("0: Write value 0 to all GPIO Pins\n");
				printk("3: Turn On all leds\n");
				printk("4: Turn Off all leds\n");
				printk("a: Turn On led of wlan0 (5G)\n");
				printk("b: Turn On led of wlan1 (2.4G)\n");
				printk("c: Turn On led of rssi0\n");
				printk("h: Show this Help Menu\n");
				break;
			default:
				printk("Invaild Value!\n");
				return -EFAULT;
		}
		return count;
	}
	return -EFAULT;
}

int led_check_single_open(struct inode *inode, struct file *file)
{
        return(single_open(file, led_check_read_proc, NULL));
}

static ssize_t led_check_single_write(struct file * file, const char __user * userbuf,
             size_t count, loff_t * off)
{
    return led_check_write_proc(file, userbuf,count, off);
}

static const struct file_operations led_check_proc_fops = {
	.open 		= led_check_single_open,
	.write 		= led_check_single_write,
	.read 		= seq_read,
	.llseek 	= seq_lseek,
	.release 	= single_release,
};

//--------------------------------------------------------
//  USB Power
//--------------------------------------------------------
static int usb_power_read_proc(struct seq_file *s, void *v)
{
	//Low Active
	// G7  G5
	// 0   0  : Enable all
	// 0   1  : Enable USB0
	// 1   0  : Enable USB1
	// 1   1  : Disable all
	switch(rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G7))
	{
		case 1:
			switch(rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G5))
			{
				case 1:
					seq_printf(s,"usb_power: 0\n"); //Disable all
					break;
				case 0:
					seq_printf(s,"usb_power: 3\n"); //Enable USB1
					break;
				default:
					seq_printf(s,"USB Power G7: %d\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G7));
					seq_printf(s,"USB Power G5: %d\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G5));
					break;
			}
			break;
		case 0:
			switch(rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G5))
			{
				case 1:
					seq_printf(s,"usb_power: 2\n"); //Enable USB0
					break;
				case 0:
					seq_printf(s,"usb_power: 1\n"); //Enable all
					break;
				default:
					seq_printf(s,"USB Power G7: %d\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G7));
					seq_printf(s,"USB Power G5: %d\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G5));
					break;
			}
			break;
		default:
			seq_printf(s,"USB Power G7: %d\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G7));
			seq_printf(s,"USB Power G5: %d\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G5));
			break;
	}
	return 0;
}

static int usb_power_write_proc(struct file *file, const char *buffer,
                unsigned long count, void *data)
{
	char usb_power;

	if (count < 2)
		return -EFAULT;

	if (buffer && !copy_from_user(&usb_power, buffer, 1)) {
		switch(usb_power)
		{//Low active
			case '1'://Enable
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G7, 0);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G5, 0);
				break;
			case '0'://Disable
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G7, 1);
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G5, 1);
				break;
			case '2':
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G7, 0); //USB0: Enable
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G5, 1); //USB1: Disable
				break;
			case '3':
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G7, 1); //USB0: Disable
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G5, 0); //USB1: Enable
				break;
			case 'd':
				printk("USB Power  G7: %d\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G7));
				printk("USB Power  G5: %d\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G5));
				break;
			case 'h':
				printk("---------------------------\n");
				printk("usb_power: Help Menu\n");
				printk("---------------------------\n");
				printk("1: Enable all USB Power\n");
				printk("0: Disable all USB Power\n");
				printk("2: Enable USB0 Power, Disable USB1 Power\n");
				printk("3: Enable USB1 Power, Disable USB0 Power\n");
				printk("h: Show this Help Menu\n");
				break;
			default:
				printk("Invaild Value!\n");
				return -EFAULT;
		}
		return count;
	}
	return -EFAULT;
}

int usb_power_single_open(struct inode *inode, struct file *file)
{
        return(single_open(file, usb_power_read_proc, NULL));
}

static ssize_t usb_power_single_write(struct file * file, const char __user * userbuf,
             size_t count, loff_t * off)
{
    return usb_power_write_proc(file, userbuf,count, off);
}

static const struct file_operations usb_power_proc_fops = {
	.open 		= usb_power_single_open,
	.write 		= usb_power_single_write,
	.read 		= seq_read,
	.llseek 	= seq_lseek,
	.release 	= single_release,
};

//--------------------------------------------------------
//  USB0 Power
//--------------------------------------------------------
static int usb0_power_read_proc(struct seq_file *s, void *v)
{
	//Low Active
	switch ( rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G7))
	{
		case 1:
			seq_printf(s,"usb0_power: 0\n");
			break;
		case 0:
			seq_printf(s,"usb0_power: 1\n");
			break;
		default:
			seq_printf(s,"USB Power G7: %d\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G7));
			break;
	}
	return 0;
}

static int usb0_power_write_proc(struct file *file, const char *buffer,
                unsigned long count, void *data)
{
	char usb0_power;

	if (count < 2)
		return -EFAULT;

	if (buffer && !copy_from_user(&usb0_power, buffer, 1)) {
		switch(usb0_power)
		{//Low active
			case '1'://Enable
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G7, 0);
				break;
			case '0'://Disable
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G7, 1);
				break;
			case 'd':
				printk("USB Power  G7: %d\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G7));
				break;
			case 'h':
				printk("---------------------------\n");
				printk("usb0_power: Help Menu\n");
				printk("---------------------------\n");
				printk("1: Enable USB0 Power\n");
				printk("0: Disable USB0 Power\n");
				printk("h: Show this Help Menu\n");
				break;
			default:
				printk("Invaild Value!\n");
				return -EFAULT;
		}
		return count;
	}
	return -EFAULT;
}

int usb0_power_single_open(struct inode *inode, struct file *file)
{
        return(single_open(file, usb0_power_read_proc, NULL));
}

static ssize_t usb0_power_single_write(struct file * file, const char __user * userbuf,
             size_t count, loff_t * off)
{
    return usb0_power_write_proc(file, userbuf,count, off);
}

static const struct file_operations usb0_power_proc_fops = {
	.open 		= usb0_power_single_open,
	.write 		= usb0_power_single_write,
	.read 		= seq_read,
	.llseek 	= seq_lseek,
	.release 	= single_release,
};

//--------------------------------------------------------
//  USB1 Power
//--------------------------------------------------------
static int usb1_power_read_proc(struct seq_file *s, void *v)
{
	//Low Active
	switch ( rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G5))
	{
		case 1:
			seq_printf(s,"usb1_power: 0\n");
			break;
		case 0:
			seq_printf(s,"usb1_power: 1\n");
			break;
		default:
			seq_printf(s,"USB Power G5: %d\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G5));
			break;
	}
	return 0;
}

static int usb1_power_write_proc(struct file *file, const char *buffer,
                unsigned long count, void *data)
{
	char usb1_power;

	if (count < 2)
		return -EFAULT;

	if (buffer && !copy_from_user(&usb1_power, buffer, 1)) {
		switch(usb1_power)
		{//Low active
			case '1'://Enable
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G5, 0);
				break;
			case '0'://Disable
				rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G5, 1);
				break;
			case 'd':
				printk("USB Power  G5: %d\n", rtl819x_gpio_pin_get_val(BSP_GPIO_PIN_G5));
				break;
			case 'h':
				printk("---------------------------\n");
				printk("usb1_power: Help Menu\n");
				printk("---------------------------\n");
				printk("1: Enable USB1 Power\n");
				printk("0: Disable USB1 Power\n");
				printk("h: Show this Help Menu\n");
				break;
			default:
				printk("Invaild Value!\n");
				return -EFAULT;
		}
		return count;
	}
	return -EFAULT;
}

int usb1_power_single_open(struct inode *inode, struct file *file)
{
        return(single_open(file, usb1_power_read_proc, NULL));
}

static ssize_t usb1_power_single_write(struct file * file, const char __user * userbuf,
             size_t count, loff_t * off)
{
    return usb1_power_write_proc(file, userbuf,count, off);
}

static const struct file_operations usb1_power_proc_fops = {
	.open 		= usb1_power_single_open,
	.write 		= usb1_power_single_write,
	.read 		= seq_read,
	.llseek 	= seq_lseek,
	.release 	= single_release,
};


static int __init rtl819x_gpio_peripheral_init(void)
{
	int err;

	rtl819x_gpio_peripheral.ngpio = BSP_GPIO_PIN_MAX;
	err = gpiochip_add(&rtl819x_gpio_peripheral);

	if (err)
		panic("cannot add rtl89x BSP_GPIO chip, error=%d", err);

	//Setting HW RESET PIN as input
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_B5);
	rtl819x_gpio_direction_input(&rtl819x_gpio_peripheral, BSP_GPIO_PIN_B5);

	//Let HW RESET PIN Enable (0:Enable, 1:Disable)
	REG32(0xb8000008) = REG32(0xb8000008) &(~(1<<5));

	//Enable watchdog
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G4);
	rtl819x_gpio_direction_output(&rtl819x_gpio_peripheral, BSP_GPIO_PIN_G4, 1);
	rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G4, 1);

	//Initial watchdog pulse
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_B3);
	rtl819x_gpio_direction_output(&rtl819x_gpio_peripheral, BSP_GPIO_PIN_B3, 0);

	//Create pulse for external watchdog
	init_timer(&watchdog_timer);
	watchdog_timer.expires = jiffies + HZ;
	watchdog_timer.data = 0;
	watchdog_timer.function = &watchdog_function;
	mod_timer(&watchdog_timer, jiffies + HZ);

	//Setup usb port 0, power disabled
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G7);
	rtl819x_gpio_direction_output(&rtl819x_gpio_peripheral, BSP_GPIO_PIN_G7, 0);
	rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G7, 1);

	//Setup usb port 1, power disabled
	rtl819x_gpio_pin_enable(BSP_GPIO_PIN_G5);
	rtl819x_gpio_direction_output(&rtl819x_gpio_peripheral, BSP_GPIO_PIN_G5, 0);
	rtl819x_gpio_pin_set_val(BSP_GPIO_PIN_G5, 1);

	proc_create_data("led_check", 0, NULL, &led_check_proc_fops,NULL);
	proc_create_data("usb_power", 0, NULL, &usb_power_proc_fops,NULL);
	proc_create_data("usb0_power", 0, NULL, &usb0_power_proc_fops,NULL);
	proc_create_data("usb1_power", 0, NULL, &usb1_power_proc_fops,NULL);
	proc_create_data("en_wdg_pulse", 0, NULL, &en_wdg_pulse_proc_fops,NULL);

	return err;
}
device_initcall(rtl819x_gpio_peripheral_init);

int gpio_to_irq(unsigned gpio)
{
    /* FIXME */
    return -EINVAL;
}
EXPORT_SYMBOL(gpio_to_irq);

int irq_to_gpio(unsigned irq)
{
    /* FIXME */
    return -EINVAL;
}
EXPORT_SYMBOL(irq_to_gpio);  

static unsigned int rtl819x_gpio_mux(u32 pin, u32 *value, u32 *address )
{
	unsigned int mask = 0;
	switch(pin) {
		case BSP_UART1_PIN:
        mask = 0xFFFFF<<0;
        *value = 0x00022222<<0;
        *address = 0xb800004c;
        break;

		// GPIOA , re-define 
		case BSP_GPIO_PIN_A0:
		/* 0xB800-0040 bit 17:16 */
		mask = 0x3<<16;
		*value = 0x3<<16;
		*address=0xb8000040;
		break;

		case BSP_GPIO_PIN_A1:
		/* 0xB800-0040 bit 13:12 */
		mask = 0x3<<12;
		*value = 0x3<<12;
		*address=0xb8000040;
		break;

		case BSP_GPIO_PIN_A2:
		case BSP_GPIO_PIN_A3:
		case BSP_GPIO_PIN_A4:
		case BSP_GPIO_PIN_A5:
		case BSP_GPIO_PIN_A6:
		/* 0xB800-004C bit 19:0 , need to be 0110 to GPIO mode */
		mask = 0xfffff<<0;
		*value = 0x66666<<0;
		*address=0xb800004c;
		break;

		case BSP_GPIO_PIN_A7:
		case BSP_GPIO_PIN_B0:
		/* 0xB800-0040 bit 5 */
		mask = 0x1<<5;
		*value = 0x1<<5;
		*address=0xb8000040;
		break;

		case BSP_GPIO_PIN_B1:
		/* 0xB800-0040 bit 6 */
		mask = 0x1<<6;
		*value = 0x1<<6;
		*address=0xb8000040;
		break;

		case BSP_GPIO_PIN_B2:
		/* 0xB800-0044 bit 2:0 */
		mask = 0x7<<0;
		*value =  0x3<<0;
		*address=0xb8000044;
		break;

		case BSP_GPIO_PIN_B3:
		/* 0xB800-0044 bit 5:3 */
		mask = 0x7<<3;
		*value =  0x3<<3;
		*address=0xb8000044;
		break;

		case BSP_GPIO_PIN_B4:
		/* 0xB800-0044 bit 8:6 */
		mask = 0x7<<6;
		*value =  0x3<<6;
		*address=0xb8000044;
		break;
		
		case BSP_GPIO_PIN_B5:
		/* 0xB800-0044 bit 11:9 */
		mask = 0x7<<9;
		*value =  0x3<<9;
		*address=0xb8000044;
		break;

		case BSP_GPIO_PIN_B6:
		/* 0xB800-0044 bit 13:12 */
		mask = 0x3<<12;
		*value =  0x3<<12;
		*address=0xb8000044;
		break;

		//case BSP_GPIO_PIN_B7:
		/* 0xB800-0044 bit 17:15 */
		//mask = 0x7<<15;
		//*value =  0x4<<15;
		//*address=0xb8000050;
		//break;

		///////////////////////////////
		//case BSP_GPIO_PIN_C0:
		/* 0xB800-0044 bit 20:18 */
		//mask = 0x7<<18;
		//*value =  0x4<<18;
		//*address=0xb8000050;
		//break;

		case BSP_GPIO_PIN_C1:
		/* 0xB800-0040 bit 21:20 */
		mask = 0x3<<20;
		*value =  0x3<<20;
		*address=0xb8000040;
		break;

		case BSP_GPIO_PIN_C2:
		/* 0xB800-0040 bit 23:22 */
		mask = 0x3<<22;
		*value =  0x3<<22;
		*address=0xb8000040;
		break;

		case BSP_GPIO_PIN_C3:
		/* 0xB800-0040 bit 25:24 */
		mask = 0x3<<24;
		*value =  0x3<<24;
		*address=0xb8000040;
		break;

		case BSP_GPIO_PIN_C4:
		/* 0xB800-0040 bit 27:26 */
		mask = 0x3<<26;
		*value =  0x3<<26;
		*address=0xb8000040;
		break;
		
		case BSP_GPIO_PIN_C5:
		/* 0xB800-0040 bit 29:28 */
		mask = 0x3<<28;
		*value =  0x3<<28;
		*address=0xb8000040;
		break;
		
		case BSP_GPIO_PIN_C6:
		/* 0xB800-0040 bit 19:18 */
		mask = 0x3<<18;
		*value =  0x3<<18;
		*address=0xb8000040;
		break;
         
		case BSP_GPIO_PIN_E0:
		case BSP_GPIO_PIN_E1:
		case BSP_GPIO_PIN_E2:
		case BSP_GPIO_PIN_E3:
		case BSP_GPIO_PIN_E4:
		case BSP_GPIO_PIN_E5:
		case BSP_GPIO_PIN_E6:
		case BSP_GPIO_PIN_E7:
		case BSP_GPIO_PIN_F0:
		case BSP_GPIO_PIN_F1:
		case BSP_GPIO_PIN_F2:
		case BSP_GPIO_PIN_F3:
		case BSP_GPIO_PIN_F4:
		/* 0xB800-0040 bit 9:7 */
		mask = 0x7<<7;
		*value =  0x3<<7;
		*address=0xb8000040;
		break;

		//case BSP_GPIO_PIN_D0: 	//?
		//case BSP_GPIO_PIN_D1: 	//?
		case BSP_GPIO_PIN_F5:
		case BSP_GPIO_PIN_F6:
		/* 0xB800-0040 bit 4:2 */
		mask = 0x7<<2;
		*value =  0x3<<2;
		*address=0xb8000040;
		break;


		
	

		case BSP_GPIO_PIN_G0:
		case BSP_GPIO_PIN_G1:
		case BSP_GPIO_PIN_G2:
		case BSP_GPIO_PIN_G3:	//?
		case BSP_GPIO_PIN_G4:	//?
		case BSP_GPIO_PIN_G5:	//?
		case BSP_GPIO_PIN_G6:	//?
		case BSP_GPIO_PIN_G7:	//?
		/* 0xB800-0040 bit 11:10 */
		mask = 0x3<<10;
		*value =  0x3<<10;
		*address=0xb8000040;
		break;
			

		default:
		break;
	}
	
	return mask;
}

