#include <linux/types.h>
#include <linux/version.h>

#include <net/sock.h>
#include <linux/skbuff.h>
#include <linux/netlink.h>

#include "8192cd_cfg.h"
#include "ieee802_mib.h"
#include "sta_proximity.h"

#if defined(STA_PROXIMITY)

struct sock *sp_nl_sock = NULL;

static size_t sp_sizeof_proximity(u_int8_t vapssidlen, u_int8_t stassidlen)
{
  size_t size = 0;

  size += nla_total_size(MACADDRLEN);
  size += nla_total_size(MACADDRLEN);
  size += nla_total_size(sizeof(u_int32_t));
  size += nla_total_size(sizeof(struct timeval));
  size += nla_total_size(sizeof(int32_t));
  size += nla_total_size(vapssidlen);
  size += nla_total_size(stassidlen);

  return size;
}


static void sp_netlink_receive(struct sk_buff *__skb)
{
       struct sk_buff *skb = NULL;
       struct nlmsghdr *nlh = NULL;
       u_int8_t *data = NULL;
       u_int32_t uid, pid, seq;

       while ((skb = skb_get(__skb)) !=NULL){
               nlh = (struct nlmsghdr *)skb->data;
               pid = NETLINK_CREDS(skb)->pid;
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0)
               uid = NETLINK_CREDS(skb)->uid;
#else
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,14,0)
               uid = NETLINK_CREDS(skb)->uid.val;
#else
#ifdef CONFIG_UIDGID_STRICT_TYPE_CHECKS
               uid = NETLINK_CREDS(skb)->uid.val;
#else
               uid = NETLINK_CREDS(skb)->uid;
#endif
#endif
#endif
               seq = nlh->nlmsg_seq;
               data = NLMSG_DATA(nlh);

               printk("recv skb from user space uid:%d pid:%d seq:%d,\n",uid,pid,seq);
               printk("data is :%s\n",(char *)data);

               //ath_sp_netlink_reply(pid, seq,data);
       }

       return ;
}


static int sp_netlink_skb(struct sk_buff *skb,
			  u_int32_t pid,
			  u_int32_t sq,
			  u_int32_t ev,
			  u_int32_t flg,
			  u_int32_t ifidx,
			  u_int8_t *vapmac,
			  u_int8_t *stamac,
			  int32_t rssi,
			  int8_t *vapssid,
			  u_int8_t vapssidlen,
			  int8_t *stassid,
			  u_int8_t stassidlen)
{
    struct nlmsghdr *nlh;
    struct timeval timestamp;

    do_gettimeofday(&timestamp);

    nlh = nlmsg_put(skb, pid, sq, ev, 0, flg);
    if (nlh == NULL)
        goto nla_put_failure;

    nla_put(skb, SP_EVENT_VAPMAC, MACADDRLEN, vapmac);
    nla_put(skb, SP_EVENT_STAMAC, MACADDRLEN, stamac);
    nla_put_u32(skb, SP_EVENT_IFINDEX, ifidx);
    nla_put(skb, SP_EVENT_RSSI, sizeof(int32_t), &rssi);
    nla_put(skb, SP_EVENT_TS, sizeof(struct timeval), &timestamp);

    if (vapssid != NULL && vapssidlen > 0)
        nla_put(skb, SP_EVENT_VAPSSID, vapssidlen, vapssid);

    if (stassid != NULL && stassidlen > 0)
        nla_put(skb, SP_EVENT_STASSID, stassidlen, stassid);

    return nlmsg_end(skb, nlh);

nla_put_failure:
    nlmsg_cancel(skb, nlh);
    return -EMSGSIZE;
}

void sp_netlink_proximity(u_int32_t ifidx,
			  u_int8_t *vapmac,
			  u_int8_t *stamac,
			  int32_t rssi,
			  int8_t *vapssid,
			  u_int8_t vapssidlen,
			  int8_t *stassid,
			  u_int8_t stassidlen)
{
    struct sk_buff *skb = NULL;
    int err = -ENOBUFS, gfp_mask;

    gfp_mask = in_interrupt() ? GFP_ATOMIC : GFP_KERNEL;
    skb = nlmsg_new(sp_sizeof_proximity(vapssidlen, stassidlen), gfp_mask);

    if (skb == NULL)
        goto errout;

    if (sp_nl_sock == NULL)
        goto errout;

    err = sp_netlink_skb(skb, 0, 0, 1, 0, ifidx, vapmac, stamac, rssi, vapssid, vapssidlen, stassid, stassidlen);
    if (err < 0)
        goto errout;

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0)
    NETLINK_CB(skb).pid = 0;
#else
    NETLINK_CB(skb).portid = 0;
#endif
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20)
    NETLINK_CB(skb).dst_pid = 0;
#endif
    NETLINK_CB(skb).dst_group = 1;

    err = netlink_broadcast(sp_nl_sock, skb, 0, 1, gfp_mask);

errout:

    if (err < 0 && err != -ESRCH) {
        printk(KERN_ALERT "%s: proximity failed (%d)\n",
                __FUNCTION__, err);
    }
}


int sp_netlink_init(void)
{
	struct netlink_kernel_cfg cfg;

	memset(&cfg, 0, sizeof(cfg));
	cfg.cb_mutex = NULL;
	cfg.groups = 1;
	cfg.input = sp_netlink_receive;
	sp_nl_sock = netlink_kernel_create(&init_net, NETLINK_SP_EVENT, &cfg);

	if (sp_nl_sock == NULL) {
		printk(KERN_ERR "Netlink: Cannot create netlink socket for sta proximity\n");
		return -EIO;
	}

        return 0;

}

int sp_netlink_delete(void)
{
    if (sp_nl_sock) {
	netlink_kernel_release(sp_nl_sock);
        sp_nl_sock = NULL;
    }

    return 0;
}

#endif /* end of #if STA_PROXIMITY */
