#include <linux/kobject.h>
#include <linux/device.h>
#include <linux/netdevice.h>
#include <linux/sysfs.h>

#include "./8192cd.h"
#include "./ieee802_mib.h"

#ifdef RTK_NL80211
#include <net/cfg80211.h>
#include "./8192cd_cfg80211.h"
#endif

#include "./sysfs_stats.h"

/* XXX: redefined already existing arrays */
static const unsigned short __HT_MCS_DATA_RATE[2][2][24] =
{
        {       {6.5, 13, 19.5, 26, 39, 52, 58.5, 65,
                 13, 26, 39 ,52, 78, 104, 117, 130,
                 19.5, 39, 58.5, 78 ,117, 156, 175.5, 195},                             // Long GI, 20MHz

                {7.2, 14.4, 21.7, 28.9, 43.3, 57.8, 65, 72.2,
                 14.4, 28.9, 43.3, 57.8, 86.7, 115.6, 130, 144.5,
                 21.7, 43.3, 65, 86.7, 130, 173.3, 195, 216.7}                   },     // Short GI, 20MHz

        {       {13.5, 27, 40.5, 54, 81, 108, 121.5, 135,
                 27, 54, 81, 108, 162, 216, 243, 270,
                 40.5, 81, 121.5, 162, 243, 324, 364.5, 405},                           // Long GI, 40MHz

                {15, 30, 45, 60, 90, 120, 135, 150,
                 30, 60, 90, 120, 180, 240, 270, 300,
                 45, 90, 135, 180, 270, 360, 405, 450}                           }      // Short GI, 40MHz
};

static const unsigned short __VHT_MCS_DATA_RATE[3][2][30] =
{
        {       {13, 26, 39, 52, 78, 104, 117, 130, 156, 156,
                 26, 52, 78, 104, 156, 208, 234, 260, 312, 312,
                 39, 78, 117, 156, 234, 312, 351, 390, 468, 520},                       // Long GI, 20MHz

                {14, 29, 43, 58, 87, 116, 130, 144, 173, 173,
                 29, 58, 87, 116, 173, 231, 260, 289, 347, 347,
                 43, 86, 130, 173, 260, 347, 390, 433, 520, 578}                 },     // Short GI, 20MHz

        {       {27, 54, 81, 108, 162, 216, 243, 270, 324, 360,
                 108, 162, 216, 324, 432, 486, 540, 648, 720,
                 162, 243, 342, 486, 648, 729, 810, 972, 1080},                         // Long GI, 40MHz

                {30, 60, 90, 120, 180, 240, 270, 300,360, 400,
                 60, 120, 180, 240, 360, 480, 540, 600, 720, 800,
                 90, 180, 270, 360, 540, 720, 810, 900, 1080, 1200}              },     // Short GI, 40MHz

        {       {59, 117,  176, 234, 351, 468, 527, 585, 702, 780,
                 117, 234, 351, 468, 702, 936, 1053, 1170, 1404, 1560,
                 176, 351, 527, 702, 1053, 1408, 1408, 1745, 2106, 2340},               // Long GI, 80MHz

                {65, 130, 195, 260, 390, 520, 585, 650, 780, 867,
                 130, 260, 390, 520, 780, 1040, 1170, 1300, 1560, 1733,
                 195, 390, 585, 780, 1170, 1560, 1560, 1950, 2340, 2600}         }      // Short GI, 80MHz
};

/* bin-radio-stats */
ssize_t rstats_read(struct file *fd, struct kobject *kobj, struct bin_attribute *attr,
                char *buf, loff_t offset, size_t size)
{
        struct device *dv;
        struct net_device *ndev;
        struct rtl8192cd_priv *priv;
        struct wifi_mib *pmib;
        struct priv_shared_info *pshare;
        struct radio_stats *rstats;

        int i;
        int tx_bw_index = 0;
        int tx_gi_index =0;
        int tx_rate_index = 0;
        unsigned int tx_rate_id = 0;
        unsigned int sum_tx_time = 0;
        unsigned int sum_rx_time = 0;
        unsigned int tx_time_ratio = 0;

        if (offset)
                return 0;

        if (size < sizeof(struct radio_stats))
                return 0;

        dv = kobj_to_dev(kobj);
        ndev = to_net_dev(dv);
        priv = GET_DEV_PRIV(ndev);
        pmib = GET_MIB(priv);
        pshare = priv->pshare;
        rstats = (struct radio_stats *)buf;

        memset(rstats, 0, sizeof(struct radio_stats));

        memcpy(rstats->ifname, ndev->name, sizeof(rstats->ifname));
        memcpy(rstats->mac, ndev->dev_addr, sizeof(rstats->mac));
        memcpy(rstats->cc, pmib->dot11dCountry.dot11CountryString, sizeof(rstats->cc));

        rstats->channel = pmib->dot11RFEntry.dot11channel;
        rstats->channel_width = !pshare->is_40m_bw ? 20 : (pshare->is_40m_bw * 40);

        rstats->txpower = priv->pshare->txpower_reg;

#ifdef RTK_NL80211
        if(pmib->dot11RFEntry.dot11channel >= 34)
                rstats->frequency = (unsigned short) ieee80211_channel_to_frequency(pmib->dot11RFEntry.dot11channel, IEEE80211_BAND_5GHZ);
        else
                rstats->frequency = (unsigned short) ieee80211_channel_to_frequency(pmib->dot11RFEntry.dot11channel, IEEE80211_BAND_2GHZ);
#endif

        rstats->ack_timeout = (unsigned char) pmib->miscEntry.ack_timeout;

#ifdef CH_LOAD_CAL
        rstats->cu_stats.ch_utililization = (priv->ext_stats.ch_utilization * 100) / 255;

        /* radio busy times */
        sum_tx_time += priv->ext_stats.tx_time;
        sum_rx_time += priv->ext_stats.rx_time;

        for (i = 0; i < RTL8192CD_NUM_VWLAN; i++) {
                if (IS_DRV_OPEN(priv->pvap_priv[i])) {
                        /* per vap busy times */
                        sum_tx_time += priv->pvap_priv[i]->ext_stats.tx_time;
                        sum_rx_time += priv->pvap_priv[i]->ext_stats.rx_time;
                }
        }

        rstats->cu_stats.ch_tx_percent = (sum_tx_time * 100) / 1000;
        rstats->cu_stats.ch_rx_percent = (sum_rx_time * 100) / 1000;

        /* RTK radio registers reports RX (IBSS and OBSS) channel utilization.
         * Which does not reflect actual situation when radio is TXing data so
         * for the proper utilization representation, TX time fraction should
         * be evaluated in equation. */

        /* WAR: Keep division precision for RTK chips without FPU support. */
        if (sum_tx_time || sum_rx_time)
                tx_time_ratio = (sum_tx_time << 10) / (sum_rx_time + sum_tx_time);
        else
                tx_time_ratio = 0;

        rstats->cu_stats.ch_utililization += ((tx_time_ratio * rstats->cu_stats.ch_tx_percent) >> 10);
        if (rstats->cu_stats.ch_utililization > 100)
                rstats->cu_stats.ch_utililization = 100;

        // prevent an unsigned wrap
        if (rstats->cu_stats.ch_utililization < (rstats->cu_stats.ch_tx_percent + rstats->cu_stats.ch_rx_percent))
                rstats->cu_stats.environment = 0;
        else
                rstats->cu_stats.environment = rstats->cu_stats.ch_utililization - (rstats->cu_stats.ch_tx_percent + rstats->cu_stats.ch_rx_percent);
#endif

        // ieeemode bitmask for 5G radio
        if (pmib->dot11BssType.net_work_type & (WIRELESS_11A)) {
                if (!(pmib->dot11BssType.net_work_type ^ (WIRELESS_11A | WIRELESS_11N | WIRELESS_11AC)))
                        rstats->ieeemode = IEEE_MODE_A | IEEE_MODE_N | IEEE_MODE_AC;
                else if (!(pmib->dot11BssType.net_work_type ^ (WIRELESS_11A | WIRELESS_11N)))
                        rstats->ieeemode = IEEE_MODE_A | IEEE_MODE_N;
                else if (!(pmib->dot11BssType.net_work_type ^ WIRELESS_11A))
                        rstats->ieeemode = IEEE_MODE_A;
        }

        // ieeemode bitmask for 2G radio
        if (pmib->dot11BssType.net_work_type & WIRELESS_11B) {
                if (!(pmib->dot11BssType.net_work_type ^ (WIRELESS_11B | WIRELESS_11G | WIRELESS_11N)))
                        rstats->ieeemode = IEEE_MODE_B | IEEE_MODE_G | IEEE_MODE_N;
        }

        /// XXX TODO: hardcoding noise
        memset(rstats->noise, -100, sizeof(rstats->noise));

        // get the number of tx/rx chains
        switch (pmib->dot11RFEntry.MIMO_TR_mode) {
                case MIMO_1T1R:
                        rstats->tx_chains = 1;
                        rstats->rx_chains = 1;
                        break;
                case MIMO_1T2R:
                        rstats->tx_chains = 1;
                        rstats->rx_chains = 2;
                        break;
                case MIMO_2T2R:
                        rstats->tx_chains = 2;
                        rstats->rx_chains = 2;
                        break;
                case MIMO_2T3R:
                        rstats->tx_chains = 2;
                        rstats->rx_chains = 3;
                        break;
                case MIMO_2T4R:
                        rstats->tx_chains = 2;
                        rstats->rx_chains = 4;
                        break;
                case MIMO_3T3R:
                        rstats->tx_chains = 3;
                        rstats->rx_chains = 3;
                        break;
                case MIMO_3T4R:
                        rstats->tx_chains = 3;
                        rstats->rx_chains = 4;
                        break;
                case MIMO_4T4R:
                        rstats->tx_chains = 4;
                        rstats->rx_chains = 4;
                        break;
        }

        /* initialize mcs indexes to 0xff
         * to not confuse memset'ed zeros with mcs0 */
        rstats->tx_vht_mcs = 0xff;
        rstats->tx_vht_nss = 0xff;
        rstats->tx_ht_mcs = 0xff;
        rstats->tx_legacy_rate = 0xff;

        rstats->tx_autorate = pmib->dot11StationConfigEntry.autoRate;

        // take runtime values if auto rate
        if (pmib->dot11StationConfigEntry.autoRate) {
                tx_gi_index = pshare->ht_current_tx_info & BIT(1) ? 1 : 0;
                tx_rate_id = pshare->current_tx_rate;

                if (tx_rate_id >= VHT_RATE_ID)
                        tx_bw_index = pshare->CurrentChannelBW;
                else if (tx_rate_id >= HT_RATE_ID)
                        tx_bw_index = pshare->ht_current_tx_info & BIT(0) ? 1 : 0;

        // take what is configured if fixed rate
        } else {
                if (pmib->dot11nConfigEntry.dot11nShortGIfor80M
                                || pmib->dot11nConfigEntry.dot11nShortGIfor40M
                                || pmib->dot11nConfigEntry.dot11nShortGIfor20M)
                        tx_gi_index = 1;
                else
                        tx_gi_index = 0;

                tx_rate_id = (unsigned int) get_rate_from_bit_value(pmib->dot11StationConfigEntry.fixedTxRate);

                if (tx_rate_id >= VHT_RATE_ID)
                        tx_bw_index = pshare->is_40m_bw;
                else if (tx_rate_id >= HT_RATE_ID)
                        tx_bw_index = pmib->dot11nConfigEntry.dot11nUse40M;
        }

        rstats->shortgi = (unsigned char) tx_gi_index;

        // ht, vht, nss indexes, legacy rate, max bitrate
        if (tx_rate_id >= VHT_RATE_ID) {
                tx_rate_index = tx_rate_id - VHT_RATE_ID;
                rstats->tx_vht_mcs = tx_rate_index % 10;
                rstats->tx_vht_nss = (tx_rate_index / 10) + 1;
                rstats->tx_datarate = __VHT_MCS_DATA_RATE[tx_bw_index][tx_gi_index][tx_rate_index] >> 1;
        } else if (tx_rate_id >= HT_RATE_ID) {
                tx_rate_index = tx_rate_id - HT_RATE_ID;
                rstats->tx_ht_mcs = tx_rate_index;
                rstats->tx_datarate = __HT_MCS_DATA_RATE[tx_bw_index][tx_gi_index][tx_rate_index];
        } else {
                rstats->tx_legacy_rate = tx_rate_id / 2;
                rstats->tx_datarate = rstats->tx_legacy_rate;
        }

        return sizeof(struct radio_stats);
}

ssize_t rstats_write(struct file *fd, struct kobject *kobj, struct bin_attribute *attr,
                char *buf, loff_t offset, size_t size)
{
        return 0;
}

struct bin_attribute rstats_attr =
__BIN_ATTR(bin-radio-stats, 0664, rstats_read, rstats_write, 0);


/* bin-vap-stats */
ssize_t vstats_read(struct file *fd, struct kobject *kobj, struct bin_attribute *attr,
                char *buf, loff_t offset, size_t size)
{
        struct device *dv;
        struct net_device *ndev;
        struct rtl8192cd_priv *priv, *proot;
        struct wifi_mib *pmib;
        struct vap_stats *vstats;

        if (offset)
                return 0;

        if (size < sizeof(struct vap_stats))
                return 0;

        dv = kobj_to_dev(kobj);
        ndev = to_net_dev(dv);
        priv = GET_DEV_PRIV(ndev);
        pmib = GET_MIB(priv);
        proot = GET_ROOT(priv);
        vstats = (struct vap_stats *)buf;

        memset(vstats, 0, sizeof(struct vap_stats));

        memcpy(vstats->ifname, ndev->name, sizeof(vstats->ifname));
        memcpy(vstats->radio_ifname, proot->dev->name, sizeof(vstats->radio_ifname));
        memcpy(vstats->mac, pmib->dot11OperationEntry.hwaddr, sizeof(vstats->mac));
        memcpy(vstats->ssid, pmib->dot11StationConfigEntry.dot11DesiredSSID, pmib->dot11StationConfigEntry.dot11DesiredSSIDLen);

        // check only certain flags
        if (OPMODE & WIFI_AP_STATE)
                vstats->opmode = WIFI_MODE_AP;
        else if (OPMODE & WIFI_STATION_STATE)
                vstats->opmode = WIFI_MODE_STA;

        vstats->hidden_ap = (unsigned char) pmib->dot11OperationEntry.hiddenAP;
        vstats->wds = (unsigned char) pmib->dot11WdsInfo.wdsEnabled;
        vstats->peer_count = (unsigned int) priv->assoc_num;
        vstats->max_peer_count = pmib->dot11StationConfigEntry.supportedStaNum;

        /* encryption */
        if (pmib->dot1180211AuthEntry.dot11PrivacyAlgrthm != _NO_PRIVACY_) {

                if ((pmib->dot1180211AuthEntry.dot11PrivacyAlgrthm == _WEP_40_PRIVACY_ ) ||
                                (pmib->dot1180211AuthEntry.dot11PrivacyAlgrthm == _WEP_104_PRIVACY_)) {

                        vstats->encryption = AUTH_WEP;

                } else if ((!pmib->dot1180211AuthEntry.dot11EnablePSK) &&
                                (pmib->dot118021xAuthEntry.dot118021xAlgrthm) &&
                                (pmib->dot1180211AuthEntry.dot11WPACipher || pmib->dot1180211AuthEntry.dot11WPA2Cipher)) {

                        if (pmib->dot1180211AuthEntry.dot11WPACipher)
                                vstats->encryption = AUTH_WPA1_EAP;

                        if (pmib->dot1180211AuthEntry.dot11WPA2Cipher)
                                vstats->encryption = AUTH_WPA2_EAP;

                        if ((pmib->dot1180211AuthEntry.dot11WPACipher) && (pmib->dot1180211AuthEntry.dot11WPA2Cipher))
                                vstats->encryption = AUTH_WPA12_EAP;

                } else if (pmib->dot1180211AuthEntry.dot11EnablePSK) {

                        if (pmib->dot1180211AuthEntry.dot11EnablePSK & BIT(0))
                                vstats->encryption = AUTH_WPA1_PSK;

                        if (pmib->dot1180211AuthEntry.dot11EnablePSK & BIT(1))
                                vstats->encryption = AUTH_WPA2_PSK;

                        if ((pmib->dot1180211AuthEntry.dot11EnablePSK & BIT(0)) && (pmib->dot1180211AuthEntry.dot11EnablePSK & BIT(1)))
                                vstats->encryption = AUTH_WPA12_PSK;
                }
        }

        /* cipher */
        if (pmib->dot1180211AuthEntry.dot11PrivacyAlgrthm == _WEP_40_PRIVACY_)
                vstats->cipher = CIPHER_WEP40;

        if (pmib->dot1180211AuthEntry.dot11PrivacyAlgrthm == _WEP_104_PRIVACY_)
                vstats->cipher = CIPHER_WEP104;

        if ((pmib->dot1180211AuthEntry.dot11WPACipher & BIT(1)) || (pmib->dot1180211AuthEntry.dot11WPA2Cipher & BIT(1)))
                vstats->cipher |= CIPHER_TKIP;

        if ((pmib->dot1180211AuthEntry.dot11WPACipher & BIT(3)) || (pmib->dot1180211AuthEntry.dot11WPA2Cipher & BIT(3)))
                vstats->cipher |= CIPHER_CCMP;


        vstats->rx_bytes = priv->net_stats.rx_bytes;
        vstats->tx_bytes = priv->net_stats.tx_bytes;
        vstats->rx_packets = priv->net_stats.rx_packets;
        vstats->tx_packets = priv->net_stats.tx_packets;
        vstats->rx_errors = priv->net_stats.rx_errors;
        vstats->tx_errors = priv->net_stats.tx_errors;

        return sizeof(struct vap_stats);
}

ssize_t vstats_write(struct file *fd, struct kobject *kobj, struct bin_attribute *attr,
                char *buf, loff_t offset, size_t size)
{
        return 0;
}

struct bin_attribute vstats_attr =
__BIN_ATTR(bin-vap-stats, 0664, vstats_read, vstats_write, 0);


/* bin-peer-stats */
ssize_t pstats_read(struct file *fd, struct kobject *kobj, struct bin_attribute *attr,
                char *buf, loff_t offset, size_t size)
{
        struct device *dv;
        struct net_device *ndev;
        struct rtl8192cd_priv *priv;
        struct wifi_mib *pmib;
        struct list_head *phead, *plist;
        struct stat_info *pstat;
        struct peer_stats *pstats;

        int tx_bw_index = 0;
        int tx_gi_index = 0;
        int tx_rate_index = 0;
        int rx_bw_index = 0;
        int rx_gi_index = 0;
        int rx_rate_index = 0;

        unsigned int tx_rate_id = 0;
        unsigned int rx_rate_id = 0;

        unsigned char *hostname;
        unsigned char *ip;
        int remaining_len;

        if (offset)
                return 0;

        if (size < sizeof(struct peer_stats))
                return 0;

        dv = kobj_to_dev(kobj);
        ndev = to_net_dev(dv);
        priv = GET_DEV_PRIV(ndev);
        pmib = GET_MIB(priv);
        pstats = (struct peer_stats *)buf;

        phead = &priv->asoc_list;
        if (!(priv->drv_state & DRV_STATE_OPEN) || list_empty(phead))
                return 0;

        plist = phead->next;
        remaining_len = size;
        while (plist != phead) {
                if (remaining_len < sizeof(struct peer_stats))
                        break;

                pstat = list_entry(plist, struct stat_info, asoc_list);

                memset(pstats, 0, sizeof(struct peer_stats));

                memcpy(pstats->ifname, ndev->name, sizeof(pstats->ifname));
                memcpy(pstats->mac, pstat->hwaddr, sizeof(pstats->mac));
                memcpy(pstats->rssi, pstat->rf_info.mimorssi, sizeof(pstats->rssi));

                fetch_proc_hostip_cache(pstat, &hostname, &ip);
                memcpy(pstats->hostip.ip, ip, sizeof(pstats->hostip.ip));
                snprintf(pstats->hostip.hostname, sizeof(pstats->hostip.hostname), "%s", hostname);

                // ieeemode bitmask
                if ((pstat->WirelessMode & WIRELESS_MODE_AC_5G) || (pstat->WirelessMode & WIRELESS_MODE_AC_24G))
                        pstats->ieeemode = IEEE_MODE_AC;
                else if (pstat->WirelessMode & WIRELESS_MODE_N_5G)
                        pstats->ieeemode = IEEE_MODE_A | IEEE_MODE_N;
                else if (pstat->WirelessMode & WIRELESS_MODE_N_24G)
                        pstats->ieeemode =  IEEE_MODE_B | IEEE_MODE_G | IEEE_MODE_N;
                else if (pstat->WirelessMode & WIRELESS_MODE_G)
                        pstats->ieeemode = IEEE_MODE_B | IEEE_MODE_G;
                else if (pstat->WirelessMode & WIRELESS_MODE_A)
                        pstats->ieeemode = IEEE_MODE_A;
                else if (pstat->WirelessMode & WIRELESS_MODE_B)
                        pstats->ieeemode = IEEE_MODE_B;

                memcpy(&pstats->assoc_time, &pstat->assoc_time, sizeof(pstats->assoc_time));

                pstats->rx_bytes = pstat->rx_bytes;
                pstats->tx_bytes = pstat->tx_bytes;
                pstats->rx_pkts= pstat->rx_pkts;
                pstats->tx_pkts = pstat->tx_pkts;

                /* initialize mcs indexes to 0xff
                 * to not confuse memset'ed zeros with mcs0 */
                pstats->tx_vht_mcs = 0xff;
                pstats->tx_vht_nss = 0xff;
                pstats->tx_ht_mcs = 0xff;
                pstats->tx_legacy_rate = 0xff;
                pstats->rx_vht_mcs = 0xff;
                pstats->rx_vht_nss = 0xff;
                pstats->rx_ht_mcs = 0xff;
                pstats->rx_legacy_rate = 0xff;

                tx_rate_id = pstat->current_tx_rate;
                tx_gi_index = pstat->ht_current_tx_info & BIT(1) ? 1 : 0;

                rx_rate_id = pstat->rx_rate;
                rx_gi_index = pstat->rx_splcp & BIT(0) ? 1 : 0;

                /* TX */
                if (tx_rate_id >= VHT_RATE_ID) {
                        tx_rate_index = tx_rate_id - VHT_RATE_ID;
                        tx_bw_index = MIN_NUM(pstat->tx_bw, 2);
                        pstats->tx_vht_mcs = tx_rate_index % 10;
                        pstats->tx_vht_nss = (tx_rate_index / 10) + 1;
                        pstats->tx_datarate = __VHT_MCS_DATA_RATE[tx_bw_index][tx_gi_index][tx_rate_index] >> 1;
                } else if (tx_rate_id >= HT_RATE_ID) {
                        tx_rate_index = tx_rate_id - HT_RATE_ID;
                        tx_bw_index = pstat->ht_current_tx_info & BIT(0) ? 1 : 0;
                        pstats->tx_ht_mcs = tx_rate_index;
                        pstats->tx_datarate = __HT_MCS_DATA_RATE[tx_bw_index][tx_gi_index][tx_rate_index];
                } else {
                        pstats->tx_legacy_rate = tx_rate_id / 2;
                        pstats->tx_datarate = pstats->tx_legacy_rate;
                }

                /* RX */
                if (rx_rate_id >= VHT_RATE_ID) {
                        rx_rate_index = rx_rate_id - VHT_RATE_ID;
                        rx_bw_index = MIN_NUM(pstat->rx_bw, 2);
                        pstats->rx_vht_mcs = rx_rate_index % 10;
                        pstats->rx_vht_nss = (rx_rate_index / 10) + 1;
                        pstats->rx_datarate = __VHT_MCS_DATA_RATE[rx_bw_index][rx_gi_index][rx_rate_index] >> 1;
                } else if (rx_rate_id >= HT_RATE_ID) {
                        rx_rate_index = rx_rate_id - HT_RATE_ID;
                        rx_bw_index = pstat->rx_bw & BIT(0) ? 1 : 0;
                        pstats->rx_ht_mcs = rx_rate_index;
                        pstats->rx_datarate = __HT_MCS_DATA_RATE[rx_bw_index][rx_gi_index][rx_rate_index];
                } else {
                        pstats->rx_legacy_rate = rx_rate_id / 2;
                        pstats->rx_datarate = pstats->rx_legacy_rate;
                }

                pstats++;
                remaining_len -= sizeof(struct peer_stats);
                plist = plist->next;
        }

        return (char *)pstats - buf;
}

ssize_t pstats_write(struct file *fd, struct kobject *kobj, struct bin_attribute *attr,
                char *buf, loff_t offset, size_t size)
{
        return 0;
}


struct bin_attribute pstats_attr =
__BIN_ATTR(bin-peer-stats, 0664, pstats_read, pstats_write, 0);


struct bin_attribute *bin_attrs[] = {
        &vstats_attr,
        &pstats_attr,
        NULL
};

struct attribute_group bin_attr_group = {
        .bin_attrs = bin_attrs
};


int rtk_sysfs_stats(struct rtl8192cd_priv *priv, struct kobject *kobj)
{
        int retval = -1;

        /* vap statistics to be collected for all wlanX and wlanX-Y interfaces except tmp.wlanX interface */
        if ((IS_VAP_INTERFACE(priv) && strncmp(kobj->name, "tmp.", 4)) || IS_ROOT_INTERFACE(priv))
                retval = sysfs_create_group(kobj, &bin_attr_group);

        if (IS_ROOT_INTERFACE(priv))
                retval = sysfs_create_bin_file(kobj, &rstats_attr);

        if (retval)
                goto error;

        return 0;

error:
        return -ENODEV;
}

void rtk_sysfs_stats_cleanup(struct rtl8192cd_priv *priv, struct kobject *kobj)
{
        sysfs_remove_group(kobj, &bin_attr_group);
        sysfs_remove_bin_file(kobj, &rstats_attr);
}

