#ifndef __SYSFS_STATS_H__
#define __SYSFS_STATS_H__

#ifdef __KERNEL__
#include <linux/kobject.h>
#include "./8192cd.h"
int rtk_sysfs_stats(struct rtl8192cd_priv *priv, struct kobject *kobj);
void rtk_sysfs_stats_cleanup(struct rtl8192cd_priv *priv, struct kobject *kobj);
#endif

struct cu_stats {
        unsigned char   ch_utililization;       // self bss and obss channel utilization
        unsigned char   environment;            // other bss channel utilization
        unsigned char   ch_tx_percent;
        unsigned char   ch_rx_percent;
};

struct radio_stats {
        char            ifname[IFNAMSIZ];
        unsigned char   mac[6];
        unsigned char   cc[3];                  // country code
        unsigned char   channel;
        unsigned short  channel_width;
        unsigned short  frequency;
        unsigned short  txpower;
        unsigned char   ack_timeout;
        unsigned char   shortgi;

        unsigned char   tx_autorate;            // 0: fixed rate; 1: auto rate;
        unsigned char   tx_vht_mcs;             // 0-9
        unsigned char   tx_vht_nss;             // 0-4
        unsigned char   tx_ht_mcs;              // 0-32
        unsigned char   tx_legacy_rate;         // 1,2,5.5,6,9,11,24... etc
        unsigned int    tx_datarate;            // configured/runtime max datarate

        unsigned char   ieeemode;               // bit0: 11a; bit1: 11b; bit2: 11g; bit3: 11n; bit4: 11ac;
        unsigned char   tx_chains;              // number of tx chains
        unsigned char   rx_chains;              // number of rx chains
        char            noise[4];               // noise for every chain

        struct cu_stats cu_stats;               // channel utilization info
};

struct vap_stats {
        char            ifname[IFNAMSIZ];
        char            radio_ifname[IFNAMSIZ]; // parent interface name
        unsigned char   mac[6];
        unsigned char   ssid[32];

        unsigned char   opmode;                 // 0: unknown; 1: master; 2: client;
        unsigned char   hidden_ap;              // don't broadcast ssid, 0: false; 1: true;
        unsigned char   wds;                    // wds enabled, 0: false; 1: true;

        unsigned int    peer_count;             // connected peer count
        unsigned int    max_peer_count;         // max supported peer number

        unsigned char   encryption;
        unsigned char   cipher;

        unsigned long   rx_bytes;
        unsigned long   tx_bytes;
        unsigned long   rx_packets;
        unsigned long   tx_packets;
        unsigned long   rx_errors;
        unsigned long   tx_errors;
};

struct peer_hostip {
        char            hostname[64];
        unsigned char   ip[4];
};

struct peer_stats {
        char                    ifname[IFNAMSIZ];
        unsigned char           mac[6];

        unsigned char           rssi[4];                // rssi for every chain
        unsigned char           ieeemode;               // bit0: 11a; bit1: 11b; bit2: 11g; bit3: 11n; bit4: 11ac;

        unsigned char           tx_vht_mcs;             // 0-9
        unsigned char           tx_vht_nss;             // 0-4
        unsigned char           tx_ht_mcs;              // 0-32
        unsigned char           tx_legacy_rate;         // 1,2,5.5,6,9,11,24... etc
        unsigned int            tx_datarate;            // runtime datarate

        unsigned char           rx_vht_mcs;             // 0-9
        unsigned char           rx_vht_nss;             // 0-4
        unsigned char           rx_ht_mcs;              // 0-32
        unsigned char           rx_legacy_rate;         // 1,2,5.5,6,9,11,24... etc
        unsigned int            rx_datarate;            // runtime datarate

        unsigned long long int	rx_bytes;
        unsigned long long int	tx_bytes;
        unsigned long           rx_pkts;
        unsigned long           tx_pkts;

        struct timespec         assoc_time;
        struct peer_hostip      hostip;                 // peer hostname and ip address
};

enum IEEE_MODE {
        IEEE_MODE_A             = 0x01,
        IEEE_MODE_B             = 0x02,
        IEEE_MODE_G             = 0x04,
        IEEE_MODE_N             = 0x08,
        IEEE_MODE_AC            = 0x10
};

enum CIPHER {
        CIPHER_NO_PRIVACY       = 0x00,
        CIPHER_WEP40            = 0x01,
        CIPHER_WEP104           = 0x02,
        CIPHER_TKIP             = 0x04,
        CIPHER_CCMP             = 0x08
};

enum ENCRYPTION {
        AUTH_OPEN               = 0x00,
        AUTH_WEP                = 0x01,
        AUTH_WPA1_PSK           = 0x02,
        AUTH_WPA1_EAP           = 0x04,
        AUTH_WPA2_PSK           = 0x08,
        AUTH_WPA2_EAP           = 0x10,
        AUTH_WPA12_PSK          = 0x20,
        AUTH_WPA12_EAP          = 0x40
};

enum WIFI_OPMODE {
        WIFI_MODE_NONE          = 0x00,
        WIFI_MODE_AP            = 0x01,
        WIFI_MODE_STA           = 0x02
};

#endif //__SYSFS_STATS_H__
